-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local EditBox = ClarityLib:NewLib("Clarity4_1.Frames.EditBox", 1)
if not EditBox then return end
-- lua api
local pairs = pairs
-- blizzard api
local IsShiftKeyDown = IsShiftKeyDown
-- libs
local Metatable = ClarityLib:GetLib("Clarity4_1.Meta.Metatable")
local ScriptLib = ClarityLib:GetLib("Clarity4_1.Meta.Scripts")
-- locals
local Public = {}
local Scripts = {}
local Private = {}
local BlizzSetNumber = nil
local BlizzSetText = nil
local IsMetatableSpliced = false


-- attaches functionality to an editbox
function EditBox.Attach(editBox)
    editBox.OldValue = editBox:IsNumeric() and 0 or ""

    if not IsMetatableSpliced then
        BlizzSetNumber = editBox.SetNumber
        BlizzSetText = editBox.SetText
        Metatable.CreateSplicedMetatable(editBox, Public)
        IsMetatableSpliced = true
    end

    Metatable.Splice(editBox, Public)
    ScriptLib.Attach(editBox)
    for k,v in pairs(Scripts) do
        editBox:HookBaseScript(k, v)
    end

    return editBox
end


-- copies the value, OldValue, and numeric state from one EditBox to another
function Public.Copy(self, from)
    local value
    if from:IsNumeric() then
        self:SetNumeric(true)
        value = from:GetNumber()
        self:SetNumber(value)
    else
        self:SetNumeric(false)
        value = from:GetText()
        self:SetText(value)
    end
    return self
end


-- sets the number of history lines allowed
-- sets a script to automatically store the line when enter is pressed
function Public.EnableHistory(self, count)
    self:SetHistoryLines(count)
    self:HookScript("OnEnterPressed", Private.HistoryOnEnterPressed)
    return self
end


-- simulates pressing enter
function Public.Enter(self)
    self:CallScript("OnEnterPressed")
    return self
end


-- returns the value regardless if the edit box is text or numeric
function Public.GetValue(self)
    if self:IsNumeric() then
        return self:GetNumber()
    end
    return self:GetText()
end


-- override of the normal SetNumber
function Public.SetNumber(self, value)
    BlizzSetNumber(self, value)
    self.OldValue = value
    return self
end


-- override of the normal SetText
function Public.SetText(self, text)
    BlizzSetText(self, text)
    self.OldValue = text
    return self
end


-- returns true if the value has changed upon the most recent enter press
function Public.IsValueChanged(self)
    return self.isValueChanged
end


-- causes all text to be selected when the EditBox comes into focus
function Scripts.OnEditFocusGained(self)
    self:HighlightText()
end


-- deselects the text when the EditBox loses focus
function Scripts.OnEditFocusLost(self)
    self:HighlightText(0, 0)
end


-- sets the value when enter is pressed
function Scripts.OnEnterPressed(self)
    self:ClearFocus()
    local value
    if self:IsNumeric() then
        value = self:GetNumber()
        BlizzSetNumber(self, value) -- necessary to exit edit box
    else -- text
        value = self:GetText()
        BlizzSetText(self, value) -- necessary to exit edit box
    end
    self.isValueChanged = (self.OldValue ~= value) and true or false
    self.OldValue = value
end


-- resets to the previous value when Escape is pressed
function Scripts.OnEscapePressed(self)
    self:ClearFocus()
    if self:IsNumeric() then
        BlizzSetNumber(self, self.OldValue)
    else -- text
        BlizzSetText(self, self.OldValue)
    end
end


-- this function is used to fix a bug where text does not show up when the EditBox is first shown
function Scripts.OnShow(self)
    self:SetCursorPosition(0)
end


-- moves to the next control when tab is pressed
-- moves to the previous control when shift-tab is pressed
-- todo: investigate tabbing to other controls
function Scripts.OnTabPressed(self)
    local nextControl
    if IsShiftKeyDown() then
        nextControl = self.PrevControl
    else
        nextControl = self.NextControl
    end
    if nextControl then
        nextControl:SetFocus()
        nextControl:HighlightText()
    end
end


function Private.HistoryOnEnterPressed(self)
    local text = self:GetText()
    if text and text:len() > 0 then
        self:AddHistoryLine(text)
    end
end
