-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local MenuButton = ClarityLib:NewLib("Clarity4_1.Frames.MenuButton", 1)
if not MenuButton then return end

-- List of Button Types
--     Default -- highly customizable button, automatically used in one of the below types are not specified
--     Cancel -- displays "Cancel", closes menu, interactive
--        required field: nil
--     IconOnly -- displays icon, no text, interactive
--        required field: Icon
--     Separator -- specialized version of IconOnly, displays horizontal bar, non-interactive
--        optional field: Icon -- will use default if not provided
--     Title -- shows title text, non-interactive
--        required field: Text

-- List of Button Attributes
-- info.Type = [nil, "Default", "Cancel", "IconOnly", "Separator", "Title"] -- indicates the base behavior of the button
-- info.Disabled = [nil, true] the button is disabled
-- info.Text = [string] -- the text of the button
-- info.FontObject = [nil, string] -- font object used to display the text
-- info.Value = [anything] -- the value of the menu when the button is selected
-- info.Func = [function](self.Info, mouseButton, isDown, old.Info) -- the function that is called when a button is clicked
-- info.Checked = [nil, true] -- check if the button has been selected
-- info.ColorCode = [string] -- "|cAARRGGBB" embedded hex value of the button text color. Only used when button is enabled
-- info.DisableColor = [string] -- "|cAARRGGBB" embedded hex value of the button text color. Only used when button is disabled
-- info.IsNotRadio = [nil, true]  --  Check the button uses radial image if false check box image if true
--        note: IsNotRadio is automatically set to true if :SetMultiSelect(true) is called
-- info.NotCheckable = [nil, true] -- button cannot be checked, info.Checked is always false / nil
-- info.Icon = [nil, string] -- texture to display in the icon portion of the button
-- info.IconTCoordLeft = [nil, number] -- left texture coord for the Icon
-- info.IconTCoordRight = [nil, number] -- right texture coord for the Icon
-- info.IconTCoordTop = [nil, number] -- top texture coord for the Icon
-- info.IconTCoordBottom = [nil, number] -- bottom texture coord for the Icon
-- info.SubMenu = [nil, true] -- indicates presence of a sub-menu, and causes ExpandArrow to be shown
-- info.HasColorSwatch = [nil, true] -- indicates presence of color swatch
--     info.R = [0.0-1.0] -- red color component for color swatch
--     info.G = [0.0-1.0] -- green color component for color swatch
--     info.B = [0.0-1.0] -- blue color component for color swatch

-- lua api
-- blizzard api
local CANCEL = CANCEL
local CLOSE = CLOSE
local CreateFrame = CreateFrame
-- libs
-- locals
local Public = {}
local Scripts = {}
local ScriptsInvisible = {}
local ButtonType = {}
local ButtonTypeNoText = { IconOnly = 1, Separator = 1 }
local Private = {}


function MenuButton.Attach(button, isContextMenu)
    if not isContextMenu then
        -- need to adjust font string positioning
        button:SetText(" ") -- have to put a single space in there to create it before repositioning it
        local fontString = button:GetFontString()
        fontString:ClearAllPoints()
        fontString:SetPoint("Left", button, "Left", 18, 0)
    end

    for k,v in pairs(Public) do
        button[k] = v
    end

    button.InvisibleButton:SetScript("OnEnter", ScriptsInvisible.OnEnter)
    button.InvisibleButton:SetScript("OnLeave", ScriptsInvisible.OnLeave)

    for k,v in pairs(Scripts) do
        button:SetScript(k, v)
    end
    return button
end


function MenuButton.Create(parent, index, isContextMenu)
    local name = parent:GetName().."Button"..index
    local button = CreateFrame("Button", name, parent, "Clarity4_1DropDownMenuButtonTemplate")

    button:SetFrameLevel(parent:GetFrameLevel() + 2)
    button:SetID(index)

    return MenuButton.Attach(button, isContextMenu)
end


-- returns the value of associated with the button
function Public.GetValue(self, onlyIfChecked)
    if onlyIfChecked then
        if self.Info and self.Info.Checked then
            return self.Info.Value or self:GetID()
        else
            return nil
        end
    end
    return (self.Info and self.Info.Value) or self:GetID()
end


function Public.LoadInfo(self, info)
    info = info or self.Info or {}
    self.Info = info

    local buttonType = ButtonType[info.Type] or ButtonType.Default
    buttonType(self, info)

    self.Background:Hide()

    self.ColorCode = info.ColorCode -- may be overriden in Private.Disabled()
    if info.Disabled then
        Private.Disabled(self, info)
    end

    if not ButtonTypeNoText[info.Type] then
        Private.SetText(self, info)
    end

    Private.SetChecked(self, info)

    if info.SubMenu then
        self.ExpandArrow:Show()
    else
        self.ExpandArrow:Hide()
    end

    if info.HasColorSwatch then
        self.ColorSwatch.NormalTexture:SetVertexColor(info.R or 0, info.G or 0, info.B or 0)
        self.ColorSwatch:Show()
    else
        self.ColorSwatch:Hide()
    end

    return Private.GetWidth(self, info)
end


function Public.ToggleChecked(self, forceValue)
    if self.Info then
        if self.Info.NotCheckable then
            self.Info.Checked = false
            return self
        end
        if forceValue ~= nil then
            self.Info.Checked = forceValue
        else
            self.Info.Checked = not self.Info.Checked
        end
    end
    return self
end


function Scripts.OnClick(self, mouseButton, isDown)
    local old = self:GetParent():Select(self:GetID())
    if self.Info.Func then
        self.Info.Func(self.Info, mouseButton, isDown, old and old.Info)
    end
    if not self:GetParent().IsMultiSelect then
        self:GetParent():Hide()
    end
    -- any updates to button state will be reflected automatically
    -- after Func() is called.  Just update self.Info, no callback
    -- is needed
    self:LoadInfo()
    if old then
        old:LoadInfo()
    end
end


function Scripts.OnDisable(self)
    self.InvisibleButton:Show()
end


function Scripts.OnEnable(self)
    self.InvisibleButton:Hide()
end


function Scripts.OnEnter(self, isPlayerAction)
    self.Background:Show()
    self:GetParent():StopCountdownToHide()
    if self.Info.SubMenu then
        self:GetParent():GetSubMenu(self:GetID(), self.Info.SubMenu)
    end
end


function Scripts.OnLeave(self, isPlayerAction)
    if not self.ExpandArrow:IsMouseOver() then
        self.Background:Hide()
        self:GetParent():StartCountdownToHide()
        if self.Info.SubMenu and self:GetParent().SubMenu then
            self:GetParent().SubMenu:Hide()
            self:GetParent().SubMenu = nil
        end
    end
end


function ScriptsInvisible.OnEnter(self, isPlayerAction)
    self:GetParent():GetParent():StopCountdownToHide()
end


function ScriptsInvisible.OnLeave(self, isPlayerAction)
    self:GetParent():GetParent():StartCountdownToHide()
end


function ButtonType.Cancel(button, info)
    button:SetDisabledFontObject(GameFontDisableSmallLeft)
    button.InvisibleButton:Hide()
    button:Enable()
    info.NotCheckable = true
    info.Func = Private.CloseMenu
    info.__Parent = button:GetParent()
    info.Text = CANCEL
end


function ButtonType.Close(button, info)
    ButtonType.Cancel(button, info)
    info.Text = CLOSE
end


function ButtonType.Default(button, info)
    button:SetDisabledFontObject(GameFontDisableSmallLeft)
    button.InvisibleButton:Hide()
    button:Enable()
end


function ButtonType.IconOnly(button, info)
    info.IconOnly = true
    button.InvisibleButton:Hide()
    button:Enable()
    if info.Icon then
        Private.SetIcon(button.Icon, info)
        button.Icon:SetPoint("Left", button, "Left", 0, 0)
    end
    button:SetText("")
end


function ButtonType.Separator(button, info)
    info.Icon = info.Icon or "Interface\\Common\\UI-TooltipDivider-Transparent"
    info.SizeY = 8
    info.NotCheckable = true
    info.IconDropDownSizeX = true
    info.Disabled = true
    ButtonType.IconOnly(button, info)
end


function ButtonType.Title(button, info)
    info.NotCheckable = true
    info.Disabled = true
    button:SetDisabledFontObject(GameFontNormalSmallLeft)
end


function Private.CloseMenu(info)
    info.__Parent:Hide()
end


function Private.Disabled(button, info)
    button:Disable()
    button.InvisibleButton:Show()
    button.ColorCode = info.DisableColor or button.ColorCode
end


function Private.GetWidth(button, info)
    local width
    if info.IconOnly and info.Icon then
        width = button.Icon:GetWidth()
    elseif button.NormalText:GetText() then
        width = button.NormalText:GetWidth() + 40
        if info.Icon then
            width = width + 10
        end
    else
        return 0
    end
    if info.SubMenu or info.HasColorSwatch then
        width = width + 10
    end
    if info.NotCheckable then
        width = width - 30
    end
    if info.Padding then
        width = width + info.Padding
    end

    return width
end


function Private.SetChecked(button, info)
    if info.NotCheckable then
        info.Checked = false
        button.Check:Hide()
        button.Uncheck:Hide()
    else
        if info.Checked then
            if info.IsNotRadio then
                button.Check:SetTexCoord(0.0, 0.5, 0.0, 0.5)
            else
                button.Check:SetTexCoord(0, 0.5, 0.5, 1)
            end
            button.Check:Show()
            button.Uncheck:Hide()
        else
            if info.IsNotRadio then
                button.Uncheck:SetTexCoord(0.5, 1.0, 0.0, 0.5)
            else
                button.Uncheck:SetTexCoord(0.5, 1, 0.5, 1)
            end
            button.Check:Hide()
            button.Uncheck:Show()
        end
    end
end


function Private.SetIcon(icon, info)
    icon:SetTexture(info.Icon)
    if info.IconTCoordLeft then
        icon:SetTexCoord(info.IconTCoordLeft, info.IconTCoordRight, info.IconTCoordTop, info.IconTCoordBottom)
    else
        icon:SetTexCoord(0, 1, 0, 1)
    end
    icon:SetWidth(info.SizeX or 16)
    icon:SetHeight(info.SizeY or 16)
    icon:Show()
end


function Private.SetText(button, info)
    if info.Text then
        if button.ColorCode then
            button:SetText(button.ColorCode..info.Text.."|r")
        else
            button:SetText(info.Text)
        end

        if info.Icon then
            button.Icon:ClearAllPoints()
            button.Icon:SetPoint("Right", button, "Right", 0, 0)
            Private.SetIcon(button.Icon, info)
        else
            button.Icon:Hide()
        end

        if info.FontObject then
			button:SetNormalFontObject(info.FontObject);
			button:SetHighlightFontObject(info.FontObject);
        else
			button:SetNormalFontObject(GameFontHighlightSmallLeft);
			button:SetHighlightFontObject(GameFontHighlightSmallLeft);
        end
    else
        button:SetText("")
        button.Icon:Hide()
    end
end
