-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local MenuList = ClarityLib:NewLib("Clarity4_1.Frames.MenuList", 1)
if not MenuList then return end
-- lua api
local Max = math.max
local ipairs = ipairs
local pairs = pairs
-- blizzard api
local Wipe = wipe
-- libs
local MenuButton = ClarityLib:GetLib("Clarity4_1.Frames.MenuButton")
-- locals
local Public = {}
local Scripts = {}
local ScriptsContextMenu = {}
local CountDownScripts = {}
local Private = {}
local MenuArray = {}
local ContextMenuArray = {}
local kDefaultCountdown = 3

-- todo: move all selection tracking into Menu.lua thus making MenuList stateless
function MenuList.GetMenu()
    -- find an existing but unused one
    for _,menu in ipairs(MenuArray) do
        if menu and not menu:IsShown() then
            return menu
        end
    end
    -- else create a new one
    local index = #MenuArray + 1
    local menu = CreateFrame("Button", "Clarity41Menu"..index, UIParent, "Clarity4_1DropDownListTemplate")
    MenuButton.Attach(menu.Buttons[1])
    menu.MenuBackDrop:Hide()
    menu.BackDrop:Show()
    MenuArray[index] = menu

    for k,v in pairs(Public) do
        menu[k] = v
    end

    for k,v in pairs(Scripts) do
        menu:SetScript(k, v)
    end

    return menu
end


function MenuList.GetContextMenu(frame, point, relativeTo, relativePoint, x, y)
    return MenuList.GetContextSubMenu(1, frame, point, relativeTo, relativePoint, x, y)
end


function MenuList.GetContextSubMenu(depth, frame, point, relativeTo, relativePoint, x, y)
    if ContextMenuArray[depth] then
        ContextMenuArray[depth]:Hide()
        Private.AttachContextMenu(frame, ContextMenuArray[depth], point, relativeTo, relativePoint, x, y)
        return ContextMenuArray[depth]
    end

    -- else create a new one
    local contextMenu = CreateFrame("Button", "Clarity41ContextMenu"..depth, UIParent, "Clarity4_1DropDownListTemplate")
    contextMenu.IsContextMenu = true
    contextMenu.Depth = depth
    MenuButton.Attach(contextMenu.Buttons[1], true)
    contextMenu.MenuBackDrop:Show()
    contextMenu.BackDrop:Hide()
    contextMenu.IsContextMenu = true

    for k,v in pairs(Public) do
        contextMenu[k] = v
    end

    for k,v in pairs(Scripts) do
        contextMenu:SetScript(k, v)
    end

    for k,v in pairs(ScriptsContextMenu) do
        contextMenu:SetScript(k, v)
    end

    ContextMenuArray[depth] = contextMenu
    Private.AttachContextMenu(frame, contextMenu, point, relativeTo, relativePoint, x, y)
    return contextMenu
end


function Public.Initialize(self, menuInfo)
    local maxWidth = 0
    self.MenuInfo = menuInfo
    self.ActiveButtons = 0
    for index,info in ipairs(menuInfo) do
        self.Buttons[index] = self.Buttons[index] or Private.AddButton(self)
        maxWidth = Max(maxWidth, self.Buttons[index]:LoadInfo(info))
        self.Buttons[index]:Show()
    end
    self.ActiveButtons = #menuInfo

    for index = self.ActiveButtons+1,#self.Buttons do
        self.Buttons[index]:Hide()
    end

    local height = (self.Buttons[1]:GetHeight()) * self.ActiveButtons + 20
    self:SetSize(maxWidth+25, height)

    for index,info in ipairs(menuInfo) do
        if info.IconDropDownSizeX then
            local button = self.Buttons[index]
            button.Icon:SetWidth(maxWidth - 5)
        end
    end
    return self
end


function Public.Select(self, index)
    if self.Menu then
        local menu = self.Menu
        if self.IsMultiSelect then
            local value = self.Buttons[index]:ToggleChecked():GetValue(true)
            menu.MultiSelection[index] = value
            menu:SetText()
        else
            local oldIndex = menu.SelectedIndex
            if oldIndex then
                oldIndex = self.Buttons[oldIndex]
                oldIndex:ToggleChecked(false)
            end
            local newButton = self.Buttons[index]:ToggleChecked(true)
            menu.SelectedIndex = index
            menu:SetText(menu.MenuInfo[index].Text or "")
            return oldIndex
        end
    end
end


function Public.GetSubMenu(self, index, subMenuInfo)
    if self.IsContextMenu then
        local subMenu = MenuList.GetContextSubMenu(self.Depth+1, self.Buttons[index], "TopLeft", self.Buttons[index], "TopRight", 8, 12)
        subMenu:Show()
        subMenu:Initialize(subMenuInfo)
        subMenu:StopCountdownToHide()
        self.SubMenu = subMenu
        subMenu.ParentMenu = self
    else

    end
end


function Public.StartCountdownToHide(self)
    if not self.IsMultiSelect then
        self.CountDown = self.TimeToHide or kDefaultCountdown
        self:SetScript("OnUpdate", CountDownScripts.OnUpdate)
    end
end


function Public.StopCountdownToHide(self)
    self.CountDown = nil
    self:SetScript("OnUpdate", nil)
end


function Scripts.OnEnter(self, isPlayerAction)
    self:StopCountdownToHide()
    if self.ParentMenu then
        self.ParentMenu:StopCountdownToHide()
    end
end


function Scripts.OnHide(self)
    -- unlock menu button
    if self.MenuButton then
        self.MenuButton:Unlock()
    end
    -- reset countdown
    self:StopCountdownToHide()
end


function ScriptsContextMenu.OnHide(self)
    if self.AttachedTo then
        self.AttachedTo.__ContextMenu = nil
        self.AttachedTo = nil
    end
    -- reset countdown
    self:StopCountdownToHide()

    if self.SubMenu then
        self.SubMenu.ParentMenu = nil
        self.SubMenu:Hide()
        self.SubMenu = nil
    end

    if self.ParentMenu then
        local parentMenu = self.ParentMenu
        parentMenu.SubMenu = nil
        self.ParentMenu = nil
        parentMenu:Initialize(parentMenu.MenuInfo)
        if not parentMenu:IsMouseOver() then
            Public.StartCountdownToHide(parentMenu)
        end
    end
end


function Scripts.OnLeave(self, isPlayerAction)
    Public.StartCountdownToHide(self)
end


function Scripts.OnShow(self)
    if not self:IsMouseOver() then
        Public.StartCountdownToHide(self)
    end
end


function CountDownScripts.OnUpdate(self, elapsed)
    if self.CountDown then
        if self.CountDown < 0 then
            if self.ParentMenu then
                local parentMenu = self.ParentMenu
                parentMenu.SubMenu = nil
                self.ParentMenu = nil
                self:Hide()
                if not parentMenu:IsMouseOver() then
                    parentMenu:Hide()
                end
            else
                self:Hide()
            end
        else
            self.CountDown = self.CountDown - elapsed
        end
    end
end


function Private.AddButton(self)
    local index = #self.Buttons + 1
    local button = MenuButton.Create(self, index, self.IsContextMenu)
    button:SetPoint("TopLeft", self.Buttons[index-1], "BottomLeft", 0, 0)
    button:SetPoint("TopRight", self.Buttons[index-1], "BottomRight", 0, 0)
    return button
end


function Private.AttachContextMenu(frame, contextMenu, point, relativeTo, relativePoint, x, y)
    contextMenu:ClearAllPoints()
    if point and relativeTo then
        contextMenu:SetPoint(point, relativeTo, relativePoint, x, y)
    end
    frame.__ContextMenu = contextMenu
    contextMenu.AttachedTo = frame
    contextMenu:SetFrameLevel(frame:GetFrameLevel() + 2)
end
