-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local VerticalScroll = ClarityLib:NewLib("Clarity4_1.Frames.VerticalScroll", 1)
if not VerticalScroll then return end
-- lua api
local Floor = math.floor
local Max = math.max
local Min = math.min
-- blizzard api
local PlaySound = PlaySound
-- libs
--local Constrain = ClarityLib:GetLib("Clarity4_1.Meta.Utility").Constrain
-- add on api
-- local name space
-- locals
local Public = {}
local Scripts = {}
local ScriptsDownButton = {}
local ScriptsUpButton = {}
local ScriptsSlider = {}

-- functions of note
-- verticalScrollFrame.ScrollBar:GetMinMaxValues()
-- verticalScrollFrame.ScrollBar:SetMinMaxValues(min, max)
-- verticalScrollFrame.ScrollBar:GetValue()
-- verticalScrollFrame.ScrollBar:SetValue()

-- sets up all the interactions of the various parts of the vertical scroll bar
function VerticalScroll.Attach(verticalScrollFrame)
    -- not using metatable or scripts lib since this is fairly light
    for k,v in pairs(Scripts) do
        verticalScrollFrame:SetScript(k, v)
    end

    verticalScrollFrame.DownButton:SetScript("OnClick", ScriptsDownButton.OnClick)

    verticalScrollFrame.UpButton:SetScript("OnClick", ScriptsUpButton.OnClick)

    verticalScrollFrame.ScrollBar:SetScript("OnValueChanged", ScriptsSlider.OnValueChanged)
    -- these two calls must be made, otherwise no thumb button will appear
    -- the actual values be in a range of 1 to n
    -- we use zeros here so that when UpdateScrollRange() is called the first time
    -- the min value will be changed to 1, and the scroll value will be changed to 1
    -- which will cause the appearance to be updated automatically
    verticalScrollFrame.ScrollBar:SetMinMaxValues(0, 100)
    verticalScrollFrame.ScrollBar:SetValue(0)

    for k,v in pairs(Public) do
        verticalScrollFrame[k] = v
    end

    verticalScrollFrame.__buttonDelta = 1
    verticalScrollFrame.__wheelDelta = 1
end


-- disables both buttons and the slider
-- the base frame does not have a built-in :Disable() function,
-- however, it has access to the OnDisable script
-- this script will never be called from the base frame
-- as it is never actually disabled
function Public.Disable(self)
    if self.__HideOnDisable then
        self:Hide()
    end
    self.__Disabled = true
    self.ScrollBar:SetMinMaxValues(1,1)
    self.ScrollBar:SetValue(1)

    self.DownButton:Disable()
    self.ScrollBar:Disable()
    self.UpButton:Disable()
end


-- enables both buttons and the slider
-- the values of the slider need to be updated after this call
-- the base frame does not have a built-in :Enable() function
-- however, it has access to the OnEnable script
-- this script will never be called from the base frame
-- as it is never actually enabled
function Public.Enable(self)
    if self.__HideOnDisable then
        self:Show()
    end
    self.__Disabled = nil
    self.DownButton:Enable()
    self.ScrollBar:Enable()
    self.UpButton:Enable()
end


function Public.Scroll(self, delta)
    local scrollPosition = self.ScrollBar:GetValue()
    scrollPosition = scrollPosition - (delta * self.__wheelDelta)
    self.ScrollBar:SetValue(scrollPosition)
end


function Public.GetScrollValue(self)
    return Floor(self.ScrollBar:GetValue() + 0.5)
end


-- anytime the VerticalScroll is updated, the callback function will be called as:
-- CallbackFunction(scrollPosition, callbackFrame)
-- the minimal callback function simply accepts the scrollPosition as a parameter
function Public.SetCallback(self, callbackFunction, callbackFrame)
    self.__callbackFunction = callbackFunction
    self.__callbackFrame = callbackFrame
end


-- for a faux scroll
--     visibleHeight: number of visible lines
--     totalHeight: total size of the data array
-- for a regular scroll
--     visibleHeight: number of visible vertical pixels
--     totalHeight: total vertical pixels
-- for both types:
--     desiredScrollValue: nil/number indicates the desired position that scroll should be set to
--         this value will be constrained by max / min of the scroll range
function Public.UpdateScrollRange(self, visibleHeight, totalHeight, desiredScrollValue)
    local lowerRange = 1
    local upperRange = totalHeight - visibleHeight + 1
    if upperRange <= lowerRange then
        self:Disable()
        self.ScrollBar:SetMinMaxValues(lowerRange, lowerRange)
        return 1
    else
        self:Enable()
        self.ScrollBar:SetMinMaxValues(lowerRange, upperRange)
        local scrollValue = desiredScrollValue or self.ScrollBar:GetValue()
        if scrollValue >= upperRange then
            self.ScrollBar:SetValue(upperRange)
            scrollValue = upperRange
            self.DownButton:Disable()
        elseif scrollValue <= lowerRange then
            self.ScrollBar:SetValue(lowerRange)
            scrollValue = lowerRange
            self.UpButton:Disable()
        else
            self.ScrollBar:SetValue(scrollValue)
        end
        return Floor(scrollValue + 0.5)
    end
end


-- part of the base frame, so the mouse wheel works over the buttons also
function Scripts.OnMouseWheel(self, delta)
    Public.Scroll(self, delta)
end


function ScriptsDownButton.OnClick(self, mouseButton, isDown)
    local verticalScrollFrame = self:GetParent()
    local scrollPosition = verticalScrollFrame.ScrollBar:GetValue()
    scrollPosition = scrollPosition + verticalScrollFrame.__buttonDelta
    verticalScrollFrame.ScrollBar:SetValue(scrollPosition)
    PlaySound("UChatScrollButton")
end


function ScriptsUpButton.OnClick(self, mouseButton, isDown)
    local verticalScrollFrame = self:GetParent()
    local scrollPosition = verticalScrollFrame.ScrollBar:GetValue()
    scrollPosition = scrollPosition - verticalScrollFrame.__buttonDelta
    verticalScrollFrame.ScrollBar:SetValue(scrollPosition)
    PlaySound("UChatScrollButton")
end


-- handles all of the updating and calls the callback
function ScriptsSlider.OnValueChanged(self, scrollPosition)
    if self.__Disabled then
        return
    end
    local verticalScrollFrame = self:GetParent()
    local min, max = self:GetMinMaxValues()
    if scrollPosition <= (min + 0.05) then
        verticalScrollFrame.UpButton:Disable()
    else
        verticalScrollFrame.UpButton:Enable()
    end
    if scrollPosition >= (max - 0.05) then
        verticalScrollFrame.DownButton:Disable()
    else
        verticalScrollFrame.DownButton:Enable()
    end
    if verticalScrollFrame.__callbackFunction then
        verticalScrollFrame.__callbackFunction(scrollPosition, verticalScrollFrame.__callbackFrame)
    end
end
