-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved

-- This takes a table and hides all fields inside of a metatable.  Upon any use of the table,
-- whether calling a function on the table, adding a new field, or even merely inspecting a
-- field, will cause the table to elvaluate.
-- IMPORTANT: the only two calls that can be made on a lazy table and not elvaluate are:
--     getmetatable(lazyObject)
--     lazyObject.IsLazy

local Lazy = ClarityLib:NewLib("Clarity4_1.Meta.Lazy", 1)
if not Lazy then return end
-- lua api
local getmetatable = getmetatable
local pairs = pairs
local setmetatable = setmetatable
-- blizzard api
-- libs
-- local name space
-- locals


-- recreates the original table
local function ActivateLazyInit(t)
    t.IsLazy = nil
    local lazyMeta = getmetatable(t)
    setmetatable(t, nil)

    local originalMeta = t.__OriginalMeta
    local originalValues = t.__OriginalValues
    local initFunction = t.__InitFunction

    t.__OriginalMeta = nil
    t.__OriginalValues = nil
    t.__InitFunction = nil

    for k,v in pairs(originalValues) do
        t[k] = v
    end
    setmetatable(t, originalMeta)
    if initFunction then
        initFunction(t)
    end
end


local lazyMeta =
{
    __index = function(t, k)
        ActivateLazyInit(t)
        return t[k]
    end,
    __newindex = function(t, k, v)
        ActivateLazyInit(t)
        t[k] = v
    end,
    __call = function(t, ...)
        ActivateLazyInit(t)
        return t(...)
    end
}


-- converts the table to a lazy table.  The first time the table is used in any way,
-- the table is reverted to its original form, and then initFunction is called, with
-- the table being its only parameter
function Lazy.MakeLazy(t, initFunction)
    local originalMeta = getmetatable(t)
    local originalValues = {}

    setmetatable(t, nil)
    for k,v in pairs(t) do
        originalValues[k] = v
        t[k] = nil
    end

    t.__OriginalMeta = originalMeta
    t.__OriginalValues = originalValues
    t.__InitFunction = initFunction
    t.IsLazy = true
    return setmetatable(t, lazyMeta)
end
