-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local Metatable = ClarityLib:NewLib("Clarity4_1.Meta.Metatable", 1)
if not Metatable then return end

-- blizzard objects have the their structure laid as follows:
--     table -- e.g. Frame, Button, etc
--         [0] UserData
--         metatable
--             __index -- table with all of the associated functions

-- this lets us splice an extra metatable into the structure so tha we can provide
-- additional functionality as well as override built in functions
--     table -- e.g. Frame, Button, etc
--          [0] UserData
--          newmetatable
--             __index -- new functions / overrides
--             metatable -- original is now metatable for newmetatable
--                 __index -- table with all of the associated functions

-- when overriding a built in function, the rest of Clarity stores the original reference
-- in a local function name so it can be used internally.  Look at EditBox.lua and search
-- for SetText to see how this is implemented

-- over-enginnering:
-- In the event that splicing in the metatable causes issues (i.e. taint), this feature can be
-- turned off.  In that case, all of the new functions and overrides are simply copied over
-- to the table as explicit key/values.

-- over-engineering: speed considerations
-- when a key is used on a table it first tries to see if it on the table it self, and failing that
-- goes to the metatable and calls __index.  If it fails there then it sees if __index has a metatable,
-- and tries that one.  This repeats until the key is found, or there is nothing more to check.  Each
-- step has an overhead cost, and thus the deeper in the metatable hierarchy we have to go, the longer
-- it takes.  So, to alleviate this, one trick is to copy everything that is not overriden into the
-- top metatable.  That way we get the (nearly) fastest speed at only minor memory overhead.


-- lua api
local getmetatable = getmetatable
local pairs = pairs
local setmetatable = setmetatable
-- blizzard api
-- libs
-- locals
local UseMetatable = true -- set to false to deactivate
local RaiseBlizzFunctions = false -- set to true to cause all functions to be raised to the top metatable


local function CreateSplicedMetatable(blizzFrame, public)
    local originalMeta = getmetatable(blizzFrame)

    -- moving all none-overridden functions up to the public table for a minute speed increase
    -- memory cost is modest, since the public table is shared amongst all of the same type
    if RaiseBlizzFunctions then
        for k,v in pairs(originalMeta.__index) do
            public[k] = public[k] or v
        end
    end

    -- used to denote the fact that we are not using the standard metatable
    public.HasClarityMeta = true

    -- allows access to overridden blizzard functions
    -- ex: frame.Blizzard.SetPoint(frame, "Center", UIParent, "Center", 0, 0)
    public.Blizzard = originalMeta.__index

    setmetatable(public, originalMeta)
    public.__index = public

    return public
end


local function Splice(blizzFrame, newMeta)
    blizzFrame.__Clarity = {}
    return setmetatable(blizzFrame, newMeta)
end


-- assigns all of the public functions directly to the frame
local function CreatePassThrough(blizzFrame, public)
    -- new functions
    for k,v in pairs(public) do
        blizzFrame[k] = v
    end

    return nil
end


-- this does almost nothing since CreatePassThrough() does
-- the actual work
local function SplicePassThrough(blizzFrame)
    blizzFrame.__Clarity = {}
    return blizzFrame
end


Metatable.CreateSplicedMetatable = CreatePassThrough
Metatable.Splice = SplicePassThrough

if UseMetatable then
    Metatable.CreateSplicedMetatable = CreateSplicedMetatable
    Metatable.Splice = Splice
end
