-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local ScriptLib = ClarityLib:NewLib("Clarity4_1.Meta.Scripts", 1)
if not ScriptLib then return end

-- Creates a new mechanism to handle scripts attached to objects.

-- Base scripts are scripts that provide some basic functionality to an object
-- and are hooked so that their functionality does not need to be constantly
-- recreated (see EditBox for the best example of this use).  These scripts may not
-- be removed from the object, though they will be disabled if the entire scriptName
-- is disabled.

-- Regular scripts are may be hooked and unhooked with no effect on the Base Scripts.  Hooking
-- a script does not remove any previous scripts.  If you want to replace one script with a new one,
-- you must unhook the olf script.

-- Furthermore, all scripts under a specific tag may be disabled and enabled, thus
-- removing the necessity of actually removing any script from the object if you wish to
-- merely suppress them.

-- The execution order is
--     Base Scripts in the order they were attached in
--     Regular Scripts in the order they were attached in


-- lua api
local getmetatable = getmetatable
local ipairs = ipairs
local pairs = pairs
local TableRemove = table.remove
-- blizzard api
-- libs
-- locals
local Public = {}
local Private = {}
local IsBlizzFunctionsSaved = false
local BlizzSetScript = nil
local BlizzHookScript = nil


-- Overrides the build in script functionality for Scriptable objects
-- If the object already has the Clarity Metatable attached to it, the
-- overrides are inserted into the metattable.  Otherwise the overrides
-- are placed directly on the object
function ScriptLib.Attach(frame)
    frame.__Clarity = frame.__Clarity or {}
    frame.__Clarity.Dispatchers = {}

    if not IsBlizzFunctionsSaved then
        if frame.HasClarityMeta then
            BlizzSetScript = frame.Blizzard.SetScript
            BlizzHookScript = frame.Blizzard.HookScript
        else
            BlizzSetScript = frame.SetScript
            BlizzHookScript = frame.HookScript
        end
        IsBlizzFunctionsSaved = true
    end

    if frame.HasClarityMeta then
        local clarityMeta = getmetatable(frame)
        for k,v in pairs(Public) do
            clarityMeta[k] = v
        end
    else
        for k,v in pairs(Public) do
            frame[k] = v
        end
    end
    return frame
end


-- helper function for to attach several scripts onto a non-clarity frame
function ScriptLib.SetAllScripts(frame, scriptsTable)
    for scriptName,scriptFunc in pairs(scriptsTable) do
        frame:SetScript(scriptName, scriptFunc)
    end
end


-- Actually hooks the function
-- if nil is passed in for the function, then the
-- scripts are simply disabled not removed
-- this is done to mimic the classic behavior / use of SetScript
function Public.SetScript(self, scriptName, scriptFunction)
    if scriptFunction then
        Public.HookScript(self, scriptName, scriptFunction)
    else
        Public.DisableScript(self, scriptName)
    end
end


-- Hooks a script in to provide basic functionality for a frame
function Public.HookBaseScript(self, scriptName, scriptFunction)
    local dispatcher = Private.GetDispatcher(self, scriptName)
    if not Private.FindInArray(dispatcher.BaseScripts, scriptFunction) then
        dispatcher.Enabled = true
        dispatcher.BaseScripts[#dispatcher.BaseScripts+1] = scriptFunction
    end
    return self
end


-- convenience function to hook everything in one line of code instead of three
function Public.HookAllBaseScripts(self, baseScriptsTable)
    for scriptName,scriptFunc in pairs(baseScriptsTable) do
        self:HookBaseScript(scriptName, scriptFunc)
    end
end


-- calls all scripts under scriptName to be executed in order of attachement
function Public.CallScript(self, scriptName, ...)
    local dispatcher = self.__Clarity.Dispatchers[scriptName]
    if dispatcher then
        dispatcher.Callback(self, ...)
        return dispatcher.Enabled
    end
    return false
end


-- clears all non-base scripts listed under scriptName
function Public.ClearScript(self, scriptName)
    if self.__Clarity.Dispatchers[scriptName] then
        self.__Clarity.Dispatchers[scriptName].Scripts = {}
    end
    return self
end


-- clears all non-base scripts
function Public.ClearAllScripts(self)
    for _,dispatcher in pairs(self.__Clarity.Dispatchers) do
        dispatcher.Scripts = {}
    end
end


-- disables the scripts under scriptName
function Public.DisableScript(self, scriptName)
    if self.__Clarity.Dispatchers[scriptName] then
        self.__Clarity.Dispatchers[scriptName].Enabled = false
    end
end


-- disables all of the scripts for the object
function Public.DisableAllScripts(self)
    for _,dispatcher in pairs(self.__Clarity.Dispatchers) do
        dispatcher.Enabled = false
    end
end


-- enables the execution of all scripts listed under scriptName
function Public.EnableScript(self, scriptName)
    if self.__Clarity.Dispatchers[scriptName] then
        self.__Clarity.Dispatchers[scriptName].Enabled = true
    end
end


-- enables the execution of all registered scripts for the object
function Public.EnableAllScripts(self)
    for _,dispatcher in pairs(self.__Clarity.Dispatchers) do
        dispatcher.Enabled = true
    end
end


-- hooks a script to be executed when scriptName is called
function Public.HookScript(self, scriptName, scriptFunction)
    local dispatcher = Private.GetDispatcher(self, scriptName)
    if not Private.FindInArray(dispatcher.Scripts, scriptFunction) then
        dispatcher.Scripts[#dispatcher.Scripts+1] = scriptFunction
    end
    dispatcher.Enabled = true
    return self
end


-- hooks a table of scripts
function Public.HookScripts(self, scriptsTable)
    for k,v in pairs(scriptsTable) do
        Public.HookScript(self, k, v)
    end
    return self
end


-- removes a script from the object
function Public.UnhookScript(self, scriptName, scriptFunction)
    local dispatcher = Private.GetDispatcher(self, scriptName)
    local index = Private.FindInArray(dispatcher.Scripts, scriptFunction)
    if index then
        TableRemove(dispatcher.Scripts, index)
    end
    return self
end


function Private.FindInArray(array, value)
    for k,v in ipairs(array) do
        if v == value then
            return k
        end
    end
    return nil
end


-- creates a dispatcher if one does not exist, and then registers it to
-- run when called
function Private.GetDispatcher(self, scriptName)
--    if not self.__Clarity then
--        print(debug.traceback())
--    end
    local dispatcher = self.__Clarity.Dispatchers[scriptName]
    if not dispatcher then
        dispatcher = Private.CreateDispatcher()
        self.__Clarity.Dispatchers[scriptName] = dispatcher
        BlizzHookScript(self, scriptName, dispatcher.Callback)
    end
    return dispatcher
end


function Private.CreateDispatcher()
    local dispatcher =
    {
        Enabled = true,
        BaseScripts = {},
        Scripts = {}
    }
    dispatcher.Callback = function(self, ...)
        if not dispatcher.Enabled then return end
        for _,script in ipairs(dispatcher.BaseScripts) do
            script(self, ...)
        end
        for _,script in ipairs(dispatcher.Scripts) do
            script(self, ...)
        end
    end
    return dispatcher
end
