-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
-- Each class using this to manage the number of displays must implement the following
-- functions:
--     :GetStateInfo()
--     :SetStateInfo(table)
--     :Initialize()
--     :Show()
--     :Hide()
-- the table returned from must contain the keys:
--     isShown
--     input
-- if the table contains noSave = true, then the entry will be deleted during Init()

-- structure
--     constructor function    called to create a new display
--     data        array       stores state info, is not sparse, slot may contain state table or false
--     displays    array       stores the displays being managed

-- warning: since the manager destroys the old stored table, only the manager should be
-- accessing it
local _, DataDump = ...
-- lua api
local ipairs = ipairs
-- blizzard api
-- libs
-- add on api
-- local name space
local DisplayManager = DataDump.NameSpace("Common", "Manager")
local DisplayMeta = {}; DisplayMeta.__index = DisplayMeta
local Private = {}
-- locals


function DisplayManager.Init(parentTable, tableKey, constructor)
    local manager = setmetatable({ constructor = constructor, displays = {} }, DisplayMeta)
    local oldDataTable = parentTable[tableKey] or {}
    local newDataTable = {}
    manager.data = newDataTable

    local newIndex = 1
    for oldIndex,data in ipairs(oldDataTable) do
        if data and not data.noSave then
            newDataTable[newIndex] = data
            newIndex = newIndex + 1
        end
    end

    parentTable[tableKey] = newDataTable
    return manager
end


function DisplayMeta.GetDisplay(self)
    local index = Private.GetNextIndex(self.data)
    return Private.GetDisplay(self, index)
end


function DisplayMeta.Save(self, display)
    local state = display:GetStateInfo()
    if not state.isShown
    and (state.noSave or not state.input or state.input:len() == 0) then
        DisplayMeta.Delete(self, display)
    else
        self.data[display.index] = state
    end
end


-- makes the slot available without messing with sparse arrays
function DisplayMeta.Delete(self, display)
    self.data[display.index] = false
end


-- forceShow = false means only displays shown during last logout will be shown
-- forceShow = true means all displays will be forced to be shown
function DisplayMeta.ShowAll(self, forceShow)
    for index,state in ipairs(self.data) do
        if state then
            if state.isShown or forceShow then
                Private.GetDisplay(self, index):Show()
            end
        end
    end
end


function DisplayMeta.HideAll(self)
    for index,state in ipairs(self.data) do
        if state and state.isShown then
            Private.GetDisplay(self, index):Hide()
        end
    end
end


-- iterate over the array until we find a false or nil value
-- performance consideration: if you have enough frames open
-- that a linear search is too slow, you have other
-- issue with bigger impact
function Private.GetNextIndex(dataTable)
    local index = 1
    while dataTable[index] do
        index = index + 1
    end
    return index
end


function Private.GetDisplay(self, index)
    local display = self.displays[index]
    if not display then
        display = self.constructor(index)
        self.displays[index] = display
    end
    if self.data[index] then
        display:SetStateInfo(self.data[index])
    else
        display:Initialize()
        DisplayMeta.Save(self, display)
    end
    return display
end
