-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
-- short instructions:
-- add DataDumpRefreshButton to a frame that needs to be repeatedly refreshed
-- Refresh.Initialize() must be called when parent frame is initialized
-- parent frame required to have a function with signature of
-- frame.RefreshData(self, forceRefresh)
-- parts that must always be updated when the RefreshButton is clicked can be forced
-- to be updated when forceRefresh == true, otherwise the function should only update
-- data that is safe to update
local _, DataDump = ...
-- lua api
local pairs = pairs
-- blizzard api
local GetTime = GetTime
-- libs
-- add on api
local L = DataDump.Localization()
-- local name space
local Refresh = DataDump.NameSpace("Common", "Refresh")
local Public = {}
local ScriptsRefresh = {}
local ScriptsRadio = {}
local ScriptsSelector = {}
local Private = {}
-- locals


function Refresh.Initialize(refresh)
    refresh:RegisterForClicks("LeftButtonUp", "RightButtonUp")

    for k,v in pairs(Public) do
        refresh[k] = v
    end

    for k,v in pairs(ScriptsRefresh) do
        refresh:SetScript(k, v)
    end

    for k,v in pairs(ScriptsSelector) do
        refresh.Selector:SetScript(k, v)
    end

    refresh.Selector.Off.Check:SetChecked(true)
    refresh.Selector.Off.Input:Hide()
    refresh.Selector.Off.Text:SetText(L["Off"])

    refresh.Selector.Frames.Check:SetChecked(false)
    refresh.Selector.Frames.Input:SetText(30)
    refresh.Selector.Frames.Text:SetText(L["FrameDelay"])

    refresh.Selector.Seconds.Check:SetChecked(false)
    refresh.Selector.Seconds.Input:SetText(1)
    refresh.Selector.Seconds.Text:SetText(L["SecondsDelay"])

    for k,v in pairs(ScriptsRadio) do
        refresh.Selector.Off.Check:SetScript(k, v)
        refresh.Selector.Frames.Check:SetScript(k, v)
        refresh.Selector.Seconds.Check:SetScript(k, v)
    end

    refresh.Selector.SetButton:SetText(L["SetRefresh"])
end


function Public.GetRefreshRate(self)
    return self.Selector.Frames.Input:GetNumber(), self.Selector.Seconds.Input:GetNumber()
end


function Public.SetRefreshRate(self, frames, seconds)
    frames = frames or 30
    seconds = seconds or 1
    self.Selector.Frames.Input:SetNumber(frames)
    self.Selector.Seconds.Input:SetNumber(seconds)
end


function Public.StopRefresh(self)
    self:SetScript("OnUpdate", nil)
    self.Normal.Anim:Stop()
    Private.AllButtonsOff(self.Selector)
    self.Selector.Off.Check:SetChecked(true)
end


function ScriptsRefresh.OnHide(self)
    Public.StopRefresh(self)
end


function ScriptsRefresh.OnClick(self, mouseButton, isDown)
    if mouseButton == "LeftButton" then
        self:GetParent():RefreshData(true)
    else
        self:SetScript("OnUpdate", nil)
        self.Normal.Anim:Stop()
        self.Selector:Show()
    end
end


function ScriptsRadio.OnClick(self, mouseButton, isDown)
    Private.AllButtonsOff(self:GetParent():GetParent())
    self:SetChecked(true)
end


function ScriptsSelector.OnHide(self)
    local refresh = self:GetParent()
    if self.Frames.Check:GetChecked() then
        local delay = self.Frames.Input:GetNumber()
        if delay > 0 then
            refresh.RefreshType = "Frame"
            refresh.Delay = delay
            refresh.NextRefresh = delay
            refresh:SetScript("OnUpdate", Private.OnUpdateFrames)
            refresh.Normal.Anim:Play()
        else
            Public.StopRefresh(refresh)
        end
    elseif self.Seconds.Check:GetChecked() then
        local delay = self.Seconds.Input:GetNumber()
        if delay > 0 then
            refresh.RefreshType = "Seconds"
            refresh.Delay = delay
            refresh.NextRefresh = GetTime() + delay
            refresh:SetScript("OnUpdate", Private.OnUpdateSeconds)
            refresh.Normal.Anim:Play()
        else
            Public.StopRefresh(refresh)
        end
    else -- off
        Public.StopRefresh(refresh)
    end
end


function Private.AllButtonsOff(selector)
    selector.Off.Check:SetChecked(false)
    selector.Frames.Check:SetChecked(false)
    selector.Seconds.Check:SetChecked(false)
end


function Private.OnUpdateFrames(self, elapsed)
    self.NextRefresh = self.NextRefresh - 1
    if self.NextRefresh < 1 then
        self.NextRefresh = self.Delay
        self:GetParent():RefreshData()
    end
end


function Private.OnUpdateSeconds(self, elapsed)
    local currentTime = GetTime()
    if currentTime >= self.NextRefresh then
        self.NextRefresh = currentTime + self.Delay
        self:GetParent():RefreshData()
    end
end
