-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Date = date
local Floor = math.floor
local TableSort = table.sort
local Time = time
local ipairs = ipairs
local pairs = pairs
local tostring = tostring
-- blizzard api
local CreateFrame = CreateFrame
local Wipe = wipe
-- libs
-- add on api
local EventTracker = DataDump.NameSpace("EventWatcher", "EventTracker")
local L = DataDump.Localization()
-- local name space
local EventScroll = DataDump.NameSpace("EventWatcher", "EventScroll")
-- locals
local _kSpacing = 2
local Public = {}
local Scripts = {}
local ScriptsLine = {}
local ScriptsHide = {}
local Private = {}


function EventScroll.Initialize(scrollFrame, events, eventsSorted, history)
    local name = scrollFrame:GetName()
    scrollFrame.Events = events
    scrollFrame.EventsSorted = eventsSorted
    scrollFrame.History = history
    scrollFrame.HiddenCount = 0

    for k,v in pairs(Public) do
        scrollFrame[k] = v
    end

    Private.InitializeLine(scrollFrame.EventLine[1])
    Public.UpdateDisplayLines(scrollFrame)

    local tooltip = CreateFrame("GameTooltip", name.."GameTooltip", scrollFrame, "GameTooltipTemplate")
	tooltip.Clear = GlueTooltip_Clear;
	tooltip.SetFont = GlueTooltip_SetFont;
	tooltip.AddLine = GlueTooltip_AddLine;
	tooltip.SetText = GlueTooltip_SetText;
	tooltip.SetOwner = GlueTooltip_SetOwner;
	tooltip.GetOwner = GlueTooltip_GetOwner;
	tooltip:SetBackdropBorderColor(1.0, 1.0, 1.0);
	tooltip:SetBackdropColor(0.09, 0.09, 0.19 );
	scrollFrame.Tooltip = tooltip

    scrollFrame.Scroll:SetCallback(Private.ScrollOnValueChanged, scrollFrame)
    scrollFrame.Scroll:UpdateScrollRange(scrollFrame.DisplayLines, 0)

    for k,v in pairs(Scripts) do
        scrollFrame:SetScript(k, v)
    end
end


function Public.Update(self, offset)
    offset = (offset or self.Scroll:GetScrollValue())
    local index = 1
    self.TooltipWasShown = false
    if self.IsChronological then
        local _, maxRange = self.Scroll.ScrollBar:GetMinMaxValues()
        local lineNumber = index + offset - 1
        local history = self.History[lineNumber]
        while history and not self.Events[history.Event].IsShown do
            lineNumber = lineNumber + 1
            history = self.History[lineNumber]
        end
        while history and Private.SetLine(self, index, lineNumber, history) do
            lineNumber = lineNumber + 1
            history = self.History[lineNumber]
            while history and not self.Events[history.Event].IsShown do
                lineNumber = lineNumber + 1
                history = self.History[lineNumber]
            end
            index = index + 1
        end
        local count = #self.History - self.HiddenCount
        if offset == maxRange then
            offset = count
        end
        self.Scroll:UpdateScrollRange(self.DisplayLines, count, offset)
    else
        local event = self.EventsSorted[index + offset - 1]
        local tracker = event and self.Events[event]
        while tracker and Private.SetLine(self, index, tracker.Count, tracker.Last) do
            event = self.EventsSorted[index + offset]
            tracker = event and self.Events[event]
            index = index + 1
        end
        self.Scroll:UpdateScrollRange(self.DisplayLines, #self.EventsSorted)
    end

    while Private.ClearLine(self, index) do
         index = index + 1
    end

    if not self.TooltipWasShown then
        self.Tooltip:Hide()
    end
end


function Public.UpdateDisplayLines(scrollFrame)
    local currentLines = #scrollFrame.EventLine
    local prevLine = scrollFrame.EventLine[currentLines]
    local totalHeight = scrollFrame:GetHeight()
    local lineHeight = prevLine:GetHeight() + _kSpacing
    local displayLines = Floor( totalHeight / lineHeight)
    scrollFrame.DisplayLines = displayLines
    local name = scrollFrame:GetName()

    for index = currentLines+1,displayLines do
        local newLine = CreateFrame("Button", name.."EventLine"..index, scrollFrame, "DataDumpEventWatcherLineTemplate")
        newLine:SetPoint("TopLeft", prevLine, "BottomLeft", 0, -_kSpacing)
        newLine:SetPoint("TopRight", prevLine, "BottomRight", 0, -_kSpacing)
        Private.InitializeLine(newLine)
        scrollFrame.EventLine[index] = newLine
        prevLine = newLine
    end
end


function Scripts.OnMouseWheel(self, delta)
    self.Scroll:Scroll(delta)
end


function ScriptsLine.OnClick(self, mouseButton, isDown)
    local shownHistory = self:GetParent().ShownHistory
    local tooltip = self:GetParent().Tooltip
    if shownHistory == self.History then
        tooltip:Hide()
        self:GetParent().ShownHistory = nil
        self:UnlockHighlight()
    else
        self:GetParent().ShownHistory = self.History
        self:LockHighlight()
        tooltip:ClearAllPoints()
        tooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 27, self:GetHeight())
        tooltip:AddLine(self.History.Event, 1.0, 1.0, 1.0)
        tooltip:AddDoubleLine("Time", Private.FormatTime(self.History.TimeStamp), 1.0, 0.82, 0.0, 1.0, 1.0, 1.0)
        if self.History.Args then
            for index,arg in ipairs(self.History.Args) do
                tooltip:AddDoubleLine("Arg "..index, tostring(arg), 1.0, 0.82, 0.0, 1.0, 1.0, 1.0)
            end
        else
            tooltip:AddLine("No Args", 1.0, 1.0, 1.0)
        end
        tooltip:Show()
        if self:GetParent().LockedLine then
            self:GetParent().LockedLine:UnlockHighlight()
        end
        self:GetParent().LockedLine = self
    end
end


function ScriptsLine.OnEnter(self, isPlayerAction)
    self.HideButton:Show()
end


function ScriptsLine.OnLeave(self, isPlayerAction)
    if not self.HideButton:IsMouseOver() then
        Private.SetHideButton(self.HideButton, self:GetParent(), self.History.Event)
    end
end


function ScriptsHide.OnClick(self, mouseButton, isDown)
    local eventName = self:GetParent().History.Event
    local scrollFrame = self:GetParent():GetParent()
    local events = scrollFrame.Events
    events[eventName].IsShown = not events[eventName].IsShown
    Private.CalculateHiddenCount(scrollFrame)
    scrollFrame:GetParent():Update()
end


-- since this button is on the edge of the parent, it is possible to leave it without
-- calling the parent's OnLeave(), so we call that and thus make sure the proper
-- show/hide state is active
function ScriptsHide.OnLeave(self, isPlayerAction)
    ScriptsLine.OnLeave(self:GetParent(), isPlayerAction)
end


function Private.InitializeLine(line)
    for k,v in pairs(ScriptsLine) do
        line:SetScript(k, v)
    end
    for k,v in pairs(ScriptsHide) do
        line.HideButton:SetScript(k, v)
    end
    line:Disable()
end


function Private.SetLine(scrollFrame, index, count, history)
    if index > scrollFrame.DisplayLines then
        return false
    end
    local line = scrollFrame.EventLine[index]
    if not line then
        return false
    end
    line:Enable()
    line.EventName:SetText(history.Event)
    line.EventCount:SetText(count)
    local timeStamp = Date("%H:%M:%S", history.TimeStamp)
    line.DateTime:SetText(timeStamp)
    line.History = history
    Private.SetHideButton(line.HideButton, scrollFrame, history.Event)
    if scrollFrame.ShownHistory == history then
        scrollFrame.ShownHistory = nil
        ScriptsLine.OnClick(line)
        scrollFrame.TooltipWasShown = true
        line:LockHighlight()
    else
        line:UnlockHighlight()
    end
    return true
end


function Private.SetHideButton(hideButton, scrollFrame, eventName)
    local isShown = scrollFrame.Events[eventName].IsShown
    if isShown then
        hideButton:Hide()
    else
        hideButton:Show()
    end
end


function Private.ClearLine(scrollFrame, index)
    local line = scrollFrame.EventLine[index]
    if not line then
        return false
    end
    line:Disable()
    line.EventName:SetText("")
    line.EventCount:SetText("")
    line.DateTime:SetText("")
    line.HideButton:Hide()
    return true
end


function Private.FormatTime(timeStamp)
    local milliSeconds = Floor(((timeStamp-Floor(timeStamp)) * 1000) + 0.5)
    return Date("%H:%M:%S", timeStamp).."."..milliSeconds
end


function Private.CalculateHiddenCount(scrollFrame)
    local hidden = 0
    for event,data in pairs(scrollFrame.Events) do
        if not data.IsShown then
            hidden = hidden + data.Count
        end
    end
    scrollFrame.HiddenCount = hidden
end


function Private.ScrollOnValueChanged(scrollPosition, scrollFrame)
    scrollFrame:Update(Floor(scrollPosition))
end
