-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Date = date
local Floor = math.floor
local TableSort = table.sort
local Time = time
local pairs = pairs
-- blizzard api
local CreateFrame = CreateFrame
local GetTime = GetTime
local Wipe = wipe
-- libs
-- add on api
local EventScroll = DataDump.NameSpace("EventWatcher", "EventScroll")
local EventTracker = DataDump.NameSpace("EventWatcher", "EventTracker")
local L = DataDump.Localization()
-- local name space
local EventWatcher = DataDump.NameSpace("EventWatcher")
local Public = {}
local Scripts = {}
local ScriptsActivateAll = {}
local ScriptsClear = {}
local ScriptsDeactivateAll = {}
local ScriptsResize = {}
local ScriptsShowChronologically = {}
local ScriptsWatchAll = {}
local Private = {}
-- locals
local _watcherArray = {}


function EventWatcher.Create()
    local index = #_watcherArray+1
    local name = "DataDumpEventWatcher"..index
    local newWatcher = CreateFrame("Frame", name, UIParent, "DataDumpEventWatcherTemplate")

    -- main frame
    newWatcher.Title:SetText(L["Events "]..index)

    newWatcher.Events = {}
    newWatcher.EventTypeCount = 0
    newWatcher.EventsSorted = {}
    newWatcher.History = {}
    newWatcher.HistoryCount = 0
    newWatcher.IsDirty = false
    newWatcher.Offset = 1
    newWatcher.TimeDiff = Time() - Floor(GetTime())

    newWatcher.WatchAllEvents.Text:SetText(L["Watch All Events"])
    for k,v in pairs(ScriptsWatchAll) do
        newWatcher.WatchAllEvents:SetScript(k, v)
    end

    newWatcher.ShowChronologically.Text:SetText(L["Show Chronologically"])
    for k,v in pairs(ScriptsShowChronologically) do
        newWatcher.ShowChronologically:SetScript(k, v)
    end

    newWatcher.Clear:SetText(L["Clear"])
    for k,v in pairs(ScriptsClear) do
        newWatcher.Clear:SetScript(k, v)
    end

    newWatcher.ActivateAll:SetText(L["Activate All"])
    for k,v in pairs(ScriptsActivateAll) do
        newWatcher.ActivateAll:SetScript(k, v)
    end

    newWatcher.DeactivateAll:SetText(L["Deactivate All"])
    for k,v in pairs(ScriptsDeactivateAll) do
        newWatcher.DeactivateAll:SetScript(k, v)
    end

    for k,v in pairs(ScriptsResize) do
        newWatcher.Resize:HookScript(k, v)
    end

    newWatcher:RegisterForDrag("LeftButton")

    -- scroll area
    EventScroll.Initialize(newWatcher.ScrollArea, newWatcher.Events, newWatcher.EventsSorted, newWatcher.History)

    newWatcher.ShowChronologically:SetChecked(true)
    ScriptsShowChronologically.OnClick(newWatcher.ShowChronologically)

    for k,v in pairs(Public) do
        newWatcher[k] = v
    end

    newWatcher:SetPoint("Center", UIParent, "Center", 0, 0)
    _watcherArray[index] = newWatcher

    for k,v in pairs(Scripts) do
        newWatcher:SetScript(k, v)
    end

    return newWatcher
end


function EventWatcher.ShowAll()
    for _,watcher in pairs(_watcherArray) do
        watcher:Show()
    end
end


function EventWatcher.HideAll()
    for _,watcher in pairs(_watcherArray) do
        watcher:Hide()
    end
end


function Public.Update(self)
    Scripts.OnEvent(self)
end


function Scripts.OnDragStart(self, mouseButton)
    self:StartMoving()
end


function Scripts.OnDragStop(self)
    self:StopMovingOrSizing()
end


-- function Scripts.OnHide(self)
--     self:UnregisterAllEvents()
--     self.WatchAllEvents:SetChecked(false)
-- end


function Scripts.OnEvent(self, event, ...)
    if event then
        self.HistoryCount = self.HistoryCount + 1
        if not self.Events[event] then
            self.Events[event] = EventTracker.Create(event)
            self.EventTypeCount = self.EventTypeCount + 1
        end
        self.History[self.HistoryCount] = self.Events[event]:AddLine(GetTime()+self.TimeDiff, ...)
        if not self.Events[event].IsShown then
            self.ScrollArea.HiddenCount = self.ScrollArea.HiddenCount + 1
        end
    end
    self.IsDirty = true
end


function Scripts.OnUpdate(self, elapsed)
    if not self.IsDirty then
        return
    end
    if not self.IsChronological and self.EventTypeCount > #self.EventsSorted then
        Wipe(self.EventsSorted)
        local index = 1
        for eventName in pairs(self.Events) do
            self.EventsSorted[index] = eventName
            index = index + 1
        end
        TableSort(self.EventsSorted)
    end
    self.ScrollArea:Update()

    self.IsDirty = false
end


function ScriptsClear.OnClick(self, mouseButton, isDown)
    local watcher = self:GetParent()
    Wipe(watcher.Events)
    watcher.EventTypeCount = 0
    Wipe(watcher.EventsSorted)
    Wipe(watcher.History)
    watcher.HistoryCount = 0
    watcher.ScrollArea.HiddenCount = 0
    watcher.IsDirty = true
end


function ScriptsActivateAll.OnClick(self, mouseButton, isDown)
    local watcher = self:GetParent()
    for eventName,tracker in pairs(watcher.Events) do
        tracker.IsShown = true
    end
    watcher.IsDirty = true
end


function ScriptsDeactivateAll.OnClick(self, mouseButton, isDown)
    local watcher = self:GetParent()
    for eventName,tracker in pairs(watcher.Events) do
        tracker.IsShown = false
    end
    watcher.IsDirty = true
end


function ScriptsWatchAll.OnClick(self, mouseButton, isDown)
    local watcherFrame = self:GetParent()
    if self:GetChecked() then
        watcherFrame:RegisterAllEvents()
    else
        watcherFrame:UnregisterAllEvents()
    end
end


function ScriptsShowChronologically.OnClick(self, mouseButton, isDown)
    local isChecked = self:GetChecked()
    local watcher = self:GetParent()
    watcher.IsChronological = isChecked
    watcher.ScrollArea.IsChronological = isChecked
    watcher.ScrollArea:Update()
    watcher.IsDirty = true
end


function ScriptsResize.OnMouseDown(self)
    local watcher = self:GetParent()
    watcher:SetScript("OnUpdate", Private.OnUpdate)
end


function ScriptsResize.OnMouseUp(self)
    local watcher = self:GetParent()
    watcher:SetScript("OnUpdate", Scripts.OnUpdate)
    watcher.ScrollArea:UpdateDisplayLines()
    watcher.ScrollArea:Update()
end


function Private.OnUpdate(self, elapsed)
    self.ScrollArea:UpdateDisplayLines()
    self.ScrollArea:Update()
    Scripts.OnUpdate(self, elapsed)
end
