-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local ipairs = ipairs
local pairs = pairs
local tostring = tostring
local type = type
-- blizzard api
local CreateFrame = CreateFrame
-- libs
-- add on api
local L = DataDump.Localization()
local Simple = DataDump.NameSpace("FrameDisplay", "Editor", "Simple")
-- local name space
local BackdropTab = DataDump.NameSpace("FrameDisplay", "Backdrop")
local Public = {}
--local Private = {}
local Backdrop = {}
-- locals
local _emptyBackdrop = { insets = {} }
local _kCallback =
{
    GetBackdropBorderColor = "SetBackdropBorderColor",
    GetBackdropColor = "SetBackdropColor",
}


function BackdropTab.Create(parent)
    local name = parent:GetName().."BackdropTab"
    local displayTab = CreateFrame("Frame", name, parent, "DataDumpBackdropTabTemplate")
    displayTab:SetAllPoints()
    displayTab:SetFrameLevel(parent:GetFrameLevel()+1)

    for k,v in pairs(Public) do
        displayTab[k] = v
    end

    Backdrop.Init(displayTab.Settings.Backdrop)
    Simple.InitializeNodes(displayTab.Settings)
    displayTab.refreshList = {}

    return displayTab
end


-- dataFrame:GetBackdrop() returns a new table every time it is called
-- so lets not have auto-refresh make a bunch of tables
function Public.LoadFrame(self, dataFrame, forceRefresh)
    if forceRefresh then
        self.dataFrame = dataFrame or self.dataFrame

        if self.dataFrame then
            if self.dataFrame ~= self.enabledDataFrame or forceRefresh then
                self.refreshList.Backdrop = self.Settings.Backdrop:SetEnable(self.dataFrame)
                Simple.GetRefreshList(self.Settings, self.dataFrame, self.refreshList)
                self.enabledDataFrame = self.dataFrame
            end
            Simple.Refresh(self.refreshList)
        end
    end
end


function Public.ClearValue(self)
    self.dataFrame = nil
    Simple.ClearAllValues(self.Settings)
end


do -- RGBA node
    local RGBA = {}


    local function RGBARefresh(node)
        if node.dataFrame then
            local r, g, b, a = node.dataFrame[node.GetCall](node.dataFrame)
            node.R:SetText(Simple.ReduceNumber4(r))
            node.G:SetText(Simple.ReduceNumber4(g))
            node.B:SetText(Simple.ReduceNumber4(b))
            node.A:SetText(Simple.ReduceNumber4(a))
        end
    end


    local function RGBASetValue(node)
        if node.dataFrame then
            local r = node.R:GetNumber()
            local g = node.G:GetNumber()
            local b = node.B:GetNumber()
            local a = node.A:GetNumber()
            node.dataFrame[node.Callback](node.dataFrame, r, g, b, a)
        end
    end


    local function RGBAClearValue(node)
        node.R:SetText("")
        node.G:SetText("")
        node.B:SetText("")
        node.A:SetText("")
        node.dataFrame = nil
    end


    local function RGBASetEnable(node, dataFrame)
        if dataFrame[node.GetCall] then
            node.Title:SetFontObject(DataDumpGameFontWhiteSmallLeft)
            node.R:Enable()
            node.G:Enable()
            node.B:Enable()
            node.A:Enable()
            node.dataFrame = dataFrame
            node.__enabled = true
            return node
        else
            RGBAClearValue(node)
            node.Title:SetFontObject(GameFontDisableSmallLeft)
            node.R:Disable()
            node.G:Disable()
            node.B:Disable()
            node.A:Disable()
            node.__enabled = false
        end
    end


    function RGBA.Init(node, funcName)
        node.SetEnable = RGBASetEnable
        node.ClearValue = RGBAClearValue
        node.Title:SetText(L["Simple"][funcName])
        node.GetCall = funcName
        node.Set:SetText(L["Set"])
        node.Callback = _kCallback[funcName]
        node.Refresh = RGBARefresh
        node.SetValue = RGBASetValue
    end


    Simple.AddNodeType("RGBA", RGBA)
end -- RGBA node


do -- GetBackdrop node
    local function BackDropWorker(node, backdrop)
        node.BgFile.Input:SetText(backdrop.bgFile or "")
        node.EdgeFile.Input:SetText(backdrop.edgeFile or "")
        node.Tile.Check:SetChecked(backdrop.tile)
        node.TileSize.Input:SetText(backdrop.tileSize or "")
        node.EdgeSize.Input:SetText(backdrop.edgeSize or "")
        node.InsetLeft.Input:SetText(backdrop.insets.left or "")
        node.InsetRight.Input:SetText(backdrop.insets.right or "")
        node.InsetTop.Input:SetText(backdrop.insets.top or "")
        node.InsetBottom.Input:SetText(backdrop.insets.bottom or "")
    end


    local function BackdropRefresh(node)
        local backdrop = node.dataFrame:GetBackdrop() or _emptyBackdrop
        BackDropWorker(node, backdrop)
    end


    local function BackdropClearValue(node)
        BackDropWorker(node, _emptyBackdrop)
        node.dataFrame = nil
    end


    local function BackdropSetValue(node)
        local backdropValue = { insets = {} }

        backdropValue.bgFile = node.BgFile.Input:GetText()
        backdropValue.edgeFile = node.EdgeFile.Input:GetText()
        backdropValue.tile = node.Tile.Check:GetChecked()
        backdropValue.tileSize = node.TileSize.Input:GetNumber()
        backdropValue.edgeSize = node.EdgeSize.Input:GetNumber()
        backdropValue.insets.left = node.InsetLeft.Input:GetNumber()
        backdropValue.insets.right = node.InsetRight.Input:GetNumber()
        backdropValue.insets.top = node.InsetTop.Input:GetNumber()
        backdropValue.insets.bottom = node.InsetBottom.Input:GetNumber()

        node.dataFrame:SetBackdrop(backdropValue)
    end


    local function BackdropDeleteValue(node)
        node.dataFrame:SetBackdrop(nil)
    end


    local function BackDropSetEnable(node, dataFrame)
        if dataFrame.GetBackdrop then
            node.Set:Enable()
            node.dataFrame = dataFrame
            return node
        else
            BackdropClearValue(node)
            node.Set:Disable()
        end
    end


    function Backdrop.Init(node)
        for nodeName,subNode in pairs(node) do
            if type(subNode) == "table" and subNode.Title then
                subNode.Title:SetText(L[nodeName])
            end
        end
        node.SetEnable = BackDropSetEnable
        node.ClearValue = BackdropClearValue
        node.Set:SetText(L["Set"])
        node.Delete:SetText(L["Delete"])
        node.Refresh = BackdropRefresh
        node.SetValue = BackdropSetValue
        node.DeleteValue = BackdropDeleteValue
    end
end -- GetBackdrop node
