-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local TableRemove = table.remove
local ipairs = ipairs
local pairs = pairs
local type = type
-- blizzard api
-- libs
-- add on api
local L = DataDump.Localization()
-- local name space
local PointEditor = DataDump.NameSpace("FrameDisplay", "Editor", "Point")
-- locals
local PublicPointLine = {}
local ScriptsPointLine = {}
local PublicPointEditor = {}
local Private = {}
local _kPointMap =
{
    TOPLEFT = "TpLt",
    TOP = "Tp",
    TOPRIGHT = "TpRt",
    LEFT = "Lt",
    CENTER = "Cn",
    RIGHT = "Rt",
    BOTTOMLEFT = "BtLt",
    BOTTOM = "Bt",
    BOTTOMRIGHT = "BtRt",
}

function PointEditor.InitializePointLine(pointLine, editor)
    pointLine.Editor = editor
    pointLine.RelativeTo:SetSize(75, 10)
    for k,v in pairs(PublicPointLine) do
        pointLine[k] = v
    end
    for k,v in pairs(ScriptsPointLine) do
        pointLine:SetScript(k, v)
    end
end


function PublicPointLine.SetValue(self, frame, pointIndex)
    local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint(pointIndex)
    self.index = pointIndex
    self.dataFrame = frame
    self.Index:SetText(pointIndex)
    self.Point:SetText(_kPointMap[point])
    self.RelativeTo:SetText(Private.GetRelativeToName(relativeTo, frame))
    self.RelativePoint:SetText(_kPointMap[relativePoint])
    local xy = "X: "..Private.TruncateNumber(xOffset)..";  Y: "..Private.TruncateNumber(yOffset)
    self.XY:SetText(xy)
    return self
end


function PublicPointLine.NewLine(self, frame, index)
    self.index = index
    self.dataFrame = frame
    self.Index:SetText("+")
    self.Point:SetText("")
    self.RelativeTo:SetText(L["AddNewPoint"])
    self.RelativePoint:SetText("")
    self.XY:SetText("")
    return self
end


function ScriptsPointLine.OnClick(self, mouseButton, isDown)
    if self.Editor:IsShown() then
        self.Editor:Hide()
    else
        local level = self:GetFrameLevel()
        self.Editor:SetFrameLevel(level + 1)
        self.Editor:SetValue(self.dataFrame, self.index)
        self.Editor:Show()
    end
end


function PointEditor.InitializePointEditor(editor)
    editor.Index.Title:SetText(L["PointIndex"])
    editor.Point.Title:SetText(L["Point"])
    editor.Point.Menu.MenuInfo = Private.CreatePointMenu()
    editor.Point.Menu:SetScale(0.75)
    editor.RelativeTo.Title:SetText(L["RelativeTo"])
    editor.RelativePoint.Title:SetText(L["RelativePoint"])
    editor.RelativePoint.Menu.MenuInfo = Private.CreatePointMenu()
    editor.RelativePoint.Menu:SetScale(0.75)
    editor.XOffset.Title:SetText(L["XOffset"])
    editor.YOffset.Title:SetText(L["YOffset"])

    editor.Delete:SetText(L["Delete"])
    editor.Cancel:SetText(L["Cancel"])
    editor.Save:SetText(L["Set"])

    for k,v in pairs(PublicPointEditor) do
        editor[k] = v
    end

    editor:Hide()
    return editor
end


function PublicPointEditor.SetValue(self, dataFrame, pointIndex)
    local ptData
    if dataFrame and pointIndex <= dataFrame:GetNumPoints() then
        ptData = { dataFrame:GetPoint(pointIndex) }
        self.Delete:Enable()
    else
        ptData = { "none", nil, "none", 0, 0 }
        self.Delete:Disable()
    end

    self.dataFrame = dataFrame
    self.index = pointIndex
    self.ptData = ptData

    self.Index.Value:SetText(pointIndex)
    self.Point.Menu:SelectValue(ptData[1])
    self.RelativeTo.Input:SetText(Private.GetRelativeToName(ptData[2], dataFrame))
    self.RelativePoint.Menu:SelectValue(ptData[3])
    self.XOffset.Input:SetText(Private.TruncateNumber(ptData[4]))
    self.YOffset.Input:SetText(Private.TruncateNumber(ptData[5]))
end


function PublicPointEditor.SavePoint(self)
    local newPoint = {}
    local index = self.Point.Menu.SelectedIndex
    newPoint[1] = self.Point.Menu.MenuInfo[index].Value
    newPoint[2] = self.RelativeTo.Input:GetText()
    index = self.RelativePoint.Menu.SelectedIndex
    newPoint[3] = index > 1 and self.Point.Menu.MenuInfo[index].Value or self.ptData[1]
    newPoint[4] = self.XOffset.Input:GetNumber()
    newPoint[5] = self.YOffset.Input:GetNumber()

    if newPoint[2] == nil
    or newPoint[2]:len() == 0
    or newPoint[2] == "Parent" then
        newPoint[2] = self.dataFrame:GetParent()
    end

    local points = Private.GetAllPoints(self.dataFrame)
    points[self.index] = newPoint
    Private.SetPointArray(self.dataFrame, points)
    self.displayTab:LoadFrame(self.dataFrame)
end


function PublicPointEditor.DeletePoint(self)
    local points = Private.GetAllPoints(self.dataFrame)
    TableRemove(points, pointIndex)
    Private.SetPointArray(self.dataFrame, points)
end


function Private.TruncateNumber(n)
    return Floor(n * 100) / 100
end


function Private.GetRelativeToName(relativeTo, frame)
    if not relativeTo or relativeTo == UIParent then
        return "UIParent"
    elseif relativeTo == frame:GetParent() then
        return "Parent"
    end
    return relativeTo:GetName()
end


function Private.SetPoint(frame, pointIndex, newPoint)
    if newPoint[2] == nil then
        newPoint[2] = frame:GetParent()
    elseif type(newPoint[2]) == "string"
    and newPoint[2] == "Parent" then
        newPoint[2] = frame:GetParent()
    end

    local points = Private.GetAllPoints(frame)
    points[pointIndex] = newPoint
    Private.SetPointArray(frame, points)
end


function Private.SetPointArray(frame, points)
    frame:ClearAllPoints()

    for index,ptData in ipairs(points) do
        frame:SetPoint(ptData[1], ptData[2], ptData[3], ptData[4], ptData[5])
    end
end


function Private.GetAllPoints(frame)
    local count = frame:GetNumPoints()
    local points = {}
    for index = 1,count do
        points[index] = { frame:GetPoint(index) }
    end
    return points
end


function Private.CreatePointMenu()
    return
    {
        { Text=L["none"], Value="none" },
        { Text=L["topleft"], Value="TOPLEFT" },
        { Text=L["top"], Value="TOP" },
        { Text=L["topright"], Value="TOPRIGHT" },
        { Text=L["left"], Value="LEFT" },
        { Text=L["center"], Value="CENTER" },
        { Text=L["right"], Value="RIGHT" },
        { Text=L["bottomleft"], Value="BOTTOMLEFT" },
        { Text=L["bottom"], Value="BOTTOM" },
        { Text=L["bottomright"], Value="BOTTOMRIGHT" },
    }
end
