-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local pairs = pairs
local tostring = tostring
-- blizzard api
-- libs
-- add on api
local L = DataDump.Localization()
-- local name space
local Simple = DataDump.NameSpace("FrameDisplay", "Editor", "Simple")
local Getter = {}
local Setter = {}
local Enable = {}
local Disable = {}
local Private = {}
-- locals
local _kStrataMap =
{
    BACKGROUND = "Back",
    LOW = "Low",
    MEDIUM = "Med",
    HIGH = "High",
    DIALOG = "Dialog",
    FULLSCREEN = "Full",
    FULLSCREEN_DIALOG = "FullDia",
    TOOLTIP = "T-Tip",
}
local _kCallback =
{
    GetAlpha = "SetAlpha",
    GetBackdropBorderColor = "SetBackdropBorderColor",
    GetBackdropColor = "SetBackdropColor",
    GetDepth = "SetDepth",
    GetDontSavePosition = "SetDontSavePosition",
    GetHeight = "SetHeight",
    GetParent = "SetParent",
    GetPropagateKeyboardInput = "SetPropagateKeyboardInput",
    GetScale = "SetScale",
    GetWidth = "SetWidth",
    IsClampedToScreen = "SetClampedToScreen",
    IsIgnoringDepth = "IgnoreDepth",
    IsKeyboardEnabled = "EnableKeyboard",
    IsMouseEnabled = "EnableMouse",
    IsMouseWheelEnabled = "EnableMouseWheel",
    IsMovable = "SetMovable",
    IsResizable = "SetResizable",
    IsShown = "SetShown",
    IsToplevel = "SetToplevel",
    IsUserPlaced = "SetUserPlaced",
}
local _nodeTypes = {}


-- This works by assuming that the name of any node where node.__dataType is defined
-- is the name of the function that will be used to inspect a frame.  This has a
-- side effect in the sense that the node's name now overrides the parent node's
-- access to the function.  This only really matters if you are trying to inspect a parent
-- node with these element types of node attached to it.
function Simple.InitializeNodes(nodes)
    for funcName,node in pairs(nodes) do
        if type(node) == "table" and node.__dataType then
            local nodeType = _nodeTypes[node.__dataType]
            if nodeType then
                nodeType.Init(node, funcName)
            end
        end
    end
end


function Simple.ClearAllValues(nodes)
    for _,node in pairs(nodes) do
        if type(node) == "table" and node.ClearValue then
            node:ClearValue()
        end
    end
end


function Simple.GetRefreshList(nodes, dataFrame, refreshList)
    for funcName,node in pairs(nodes) do
        if type(node) == "table" and node.SetEnable then
            refreshList[funcName] = node:SetEnable(dataFrame)
        end
    end
    return refreshList
end


function Simple.DisableAll(nodes)
    for funcName,node in pairs(nodes) do
        if type(node) == "table" and node.Disable then
            node:Disable()
        end
    end
end


function Simple.Refresh(refreshList)
    for _,node in pairs(refreshList) do
        node:Refresh()
    end
end


function Simple.AddNodeType(nodeName, nodeTable)
    if _nodeTypes[nodeName] then
        error("'"..nodeName.."' already exists")
    end
    _nodeTypes[nodeName] = nodeTable
end


function Simple.ReduceNumber2(n)
    return (Floor(((n or 0) * 100) + 0.5)) / 100
end


function Simple.ReduceNumber4(n)
    return (Floor(((n or 0) * 10000) + 0.5)) / 10000
end


do -- ReadOnly
    local ReadOnly = {}


    local Refresh = {}
    function Refresh.Default(value)
        value:SetText(tostring(value.dataFrame[value.GetCall](value.dataFrame)))
    end


    function Refresh.GetFrameStrata(value)
        value:SetText(_kStrataMap[value.dataFrame:GetFrameStrata()])
    end


    -- attached to table so ReadOnly2Digit can reuse it
    function ReadOnly.ClearValue(node)
        node.Value:SetText("")
        node.Value.dataFrame = nil
    end


    -- attached to table so ReadOnly2Digit can reuse it
    function ReadOnly.Disable(node)
        node.Title:SetFontObject(GameFontDisableSmallLeft)
        node.Value:SetFontObject(GameFontDisableSmallLeft)
    end


    -- attached to table so ReadOnly2Digit can reuse it
    function ReadOnly.SetEnable(node, dataFrame)
        if dataFrame[node.Value.GetCall] then
            node.Title:SetFontObject(DataDumpGameFontWhiteSmallLeft)
            node.Value:SetFontObject(GameFontWhiteSmall)
            node.Value.dataFrame = dataFrame
            return node.Value
        else
            ReadOnly.ClearValue(node)
            ReadOnly.Disable(node)
        end
    end


    function ReadOnly.Init(node, funcName)
        node.SetEnable = ReadOnly.SetEnable
        node.ClearValue = ReadOnly.ClearValue
        node.Disable = ReadOnly.Disable
        node.Title:SetText(L["Simple"][funcName])
        node.Value.GetCall = funcName
        node.Value.Refresh = Refresh[funcName] or Refresh.Default
    end


    Simple.AddNodeType("ReadOnly", ReadOnly)
end -- ReadOnly


do -- ReadOnly2Digit
    local ReadOnly2Digit = {}


    local function ReadOnly2DigitRefresh(value)
        value:SetText(Simple.ReduceNumber2(value.dataFrame[value.GetCall](value.dataFrame)))
    end


    function ReadOnly2Digit.Init(node, funcName)
        node.SetEnable = _nodeTypes.ReadOnly.SetEnable
        node.ClearValue = _nodeTypes.ReadOnly.ClearValue
        node.Disable = _nodeTypes.ReadOnly.Disable
        node.Title:SetText(L["Simple"][funcName])
        node.Value.GetCall = funcName
        node.Value.Refresh = ReadOnly2DigitRefresh
    end


    Simple.AddNodeType("ReadOnly2Digit", ReadOnly2Digit)
end -- ReadOnly2Digit


do -- Bool
    local Bool = {}


    local function BoolClearValue(node)
        node.Check:SetChecked(false)
        node.Check.dataFrame = nil
    end


    local function BoolDisable(node)
        node.Title:SetFontObject(GameFontDisableSmallLeft)
        node.Check:Disable()
    end


    local function BoolSetEnable(node, dataFrame)
        if dataFrame[node.Check.GetCall] then
            node.Title:SetFontObject(DataDumpGameFontWhiteSmallLeft)
            node.Check:Enable()
            node.Check.dataFrame = dataFrame
            return node.Check
        else
            BoolClearValue(node)
            BoolDisable(node)
        end
    end


    local function BoolRefresh(check)
        check:SetChecked(check.dataFrame[check.GetCall](check.dataFrame))
    end


    function Bool.Init(node, funcName)
        node.SetEnable = BoolSetEnable
        node.ClearValue = BoolClearValue
        node.Disable = BoolDisable
        node.Title:SetText(L["Simple"][funcName])
        node.Check.GetCall = funcName
        node.Check.Callback = _kCallback[funcName]
        node.Check.Refresh = BoolRefresh
    end


    Simple.AddNodeType("Bool", Bool)
end -- Bool


do -- Edit
    local Edit = {}
    local Refresh = {}


    -- added to table so Edit2Digit can reuse it
    function Edit.ClearValue(node)
        node.Input:SetText("")
        node.Input.dataFrame = nil
    end


    -- added to table so Edit2Digit can reuse it
    function Edit.Disable(node)
        node.Title:SetFontObject(GameFontDisableSmallLeft)
        node.Input:Disable()
    end

    -- added to table so Edit2Digit can reuse it
    function Edit.SetEnable(node, dataFrame)
        if dataFrame[node.Input.GetCall] then
            node.Title:SetFontObject(DataDumpGameFontWhiteSmallLeft)
            node.Input:Enable()
            node.Input.dataFrame = dataFrame
            return node.Input
        else
            Edit.ClearValue(node)
            Edit.Disable(node)
        end
    end


    function Refresh.Default(input)
        input:SetText(tostring(input.dataFrame[input.GetCall](input.dataFrame)))
    end


    function Refresh.GetParent(input)
        local parent = input.dataFrame:GetParent()
        local name = parent and parent:GetName() or nil
        input:SetText(tostring(name))
    end


    function Edit.Init(node, funcName)
        node.SetEnable = Edit.SetEnable
        node.ClearValue = Edit.ClearValue
        node.Disable = Edit.Disable
        node.Title:SetText(L["Simple"][funcName])
        node.Input.GetCall = funcName
        node.Input.Callback = _kCallback[funcName]
        node.Input.Refresh = Refresh[funcName] or Refresh.Default
    end


    Simple.AddNodeType("Edit", Edit)
end -- Edit


do -- Edit2Digit
    local Edit2Digit = {}


    local function Edit2DigitRefresh(input)
        input:SetText(Simple.ReduceNumber2(input.dataFrame[input.GetCall](input.dataFrame)))
    end


    function Edit2Digit.Init(node, funcName)
        node.SetEnable = _nodeTypes.Edit.SetEnable
        node.ClearValue = _nodeTypes.Edit.ClearValue
        node.Disable = _nodeTypes.Edit.Disable
        node.Title:SetText(L["Simple"][funcName])
        node.Input.GetCall = funcName
        node.Input.Callback = _kCallback[funcName]
        node.Input.Refresh = Edit2DigitRefresh
        node.Input.isNumber = true -- editBox:SetNumeric(true) prevents things like decimal points, so is too restrictive
    end


    Simple.AddNodeType("Edit2Digit", Edit2Digit)
end -- Edit2Digit


do -- EditButton
    local EditButton = {}


    local function EditButtonClearValue(node)
        node.Value:SetText("")
    end


    local function EditButtonSetEnable(node, dataFrame)
        if dataFrame[node.GetCall] then
            node:Enable()
            node.dataFrame = dataFrame
            return node
        else
            EditButtonClearValue(node)
            node:Disable()
        end
    end


    local function EditButtonRefresh(node)
        node.Value:SetText(tostring(node.dataFrame[node.GetCall](node.dataFrame)))
        if node.Editor and node.Editor:IsShown() and node.Editor.Refresh then
            node.Editor:Refresh()
        end
    end


    function EditButton.Init(node, funcName)
        node.SetEnable = EditButtonSetEnable
        node.ClearValue = EditButtonClearValue
        node:SetText(L["Simple"][funcName])
        node.GetCall = funcName
        node.Refresh = EditButtonRefresh
    end


    Simple.AddNodeType("EditButton", EditButton)
end -- EditButton
