-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local pairs = pairs
local type = type
-- blizzard api
local CreateFrame = CreateFrame
-- libs
local LazyLib = ClarityLib:GetLib("Clarity4_1.Meta.Lazy")
local TabFrame = ClarityLib:GetLib("Clarity4_1.Frames.TabFrame")
-- add on api
local BackdropTab = DataDump.NameSpace("FrameDisplay", "Backdrop")
local ChildrenTab = DataDump.NameSpace("FrameDisplay", "Children")
local DisplayManager = DataDump.NameSpace("Common", "Manager")
local L = DataDump.Localization()
local Refresh = DataDump.NameSpace("Common", "Refresh")
local RegionTab = DataDump.NameSpace("FrameDisplay", "Region")
local ScriptsTab = DataDump.NameSpace("FrameDisplay", "Scripts")
local VarData = DataDump.NameSpace("VarDisplay", "Data")
-- local name space
local FrameDisplay = DataDump.NameSpace("FrameDisplay", "Display")
local Public = {}
local Scripts = {}
local ScriptsDelete = {}
local ScriptsFrameInput = {}
local ScriptsDisplayTabs = {}
local Private = {}
-- locals


function Private.Create(index)
    local name = "DataDumpFrameDisplay"..index
    local newDisplay = CreateFrame("Frame", name, UIParent, "DataDumpFrameDisplayTemplate")

    newDisplay.index = index

    newDisplay.Title:SetText(L["UIObject Display "]..index)

    for k,v in pairs(Public) do
        newDisplay[k] = v
    end

    Refresh.Initialize(newDisplay.RefreshButton)

    -- tab frame
    TabFrame.Attach(newDisplay.DisplayBase)
    newDisplay.DisplayBase:AddTab(L["Region"], RegionTab.Create(newDisplay.DisplayBase))
    newDisplay.DisplayBase:AddTab(L["Scripts"], ScriptsTab.Create(newDisplay.DisplayBase))
    newDisplay.DisplayBase:AddTab(L["Backdrop"], BackdropTab.Create(newDisplay.DisplayBase))
    newDisplay.DisplayBase:ResizeTabs(0):SelectTab(1)
    for _,tab in pairs(newDisplay.DisplayBase.Frames) do
        tab:SetScript("OnShow", ScriptsDisplayTabs.OnShow)
    end

    newDisplay:RegisterForDrag("LeftButton")
    for k,v in pairs(Scripts) do
        newDisplay:SetScript(k, v)
    end

    for k,v in pairs(ScriptsFrameInput) do
        newDisplay.FrameInput:HookScript(k, v)
    end

    -- delete button
    newDisplay.Delete:SetText(L["Delete"])
    for k,v in pairs(ScriptsDelete) do
        newDisplay.Delete:SetScript(k, v)
    end

    return newDisplay
end


function Public.SetStateInfo(self, state)
    self.FrameInput:SetText(state.input)

    local screenX, screenY = UIParent:GetSize()
    local x = Floor(state.x * screenX / 100)
    local y = Floor(state.y * screenY / 100)
    self:ClearAllPoints()
    self:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y)

    self.RefreshButton:SetRefreshRate(state.frames, state.seconds)
end


function Public.GetStateInfo(self)
    self.state = self.state or {}

    self.state.isShown = self:IsShown()
    self.state.noSave = self.noSave
    self.state.input = self.FrameInput:GetText()

    local screenX, screenY = UIParent:GetSize()
    local x, y = self:GetCenter()
    self.state.x = Floor(x * 100 / screenX)
    self.state.y = Floor(y * 100 / screenY)

    local frames, seconds = self.RefreshButton:GetRefreshRate()
    self.state.frames = frames ~= 30 and frames or nil
    self.state.seconds = seconds ~= 1 and seconds or nil

    return self.state
end


function Public.Initialize(self)
    self.FrameInput:Enable()
    self.noSave = nil
    self:ClearValue()

    self:ClearAllPoints()
    self:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
end


function Public.SetFrameInput(self, name, dataFrame)
    self.FrameInput:SetText(name)
    if name == L["•unnamed"] then
        self.FrameInput:Disable()
        self.dataFrame = dataFrame
        self.noSave = true
    end
end


function Public.RefreshData(self, forceRefresh)
    if forceRefresh then
        local frame = self.FrameInput:GetText()
        if frame and frame:len() > 0 and frame ~= L["•unnamed"] then
            if not _G[frame] then
                local parsed = VarData.Create(frame):Eval()
                frame = parsed.value
            else
                frame = _G[frame]
            end
            if frame and type(frame) == "table" and frame.IsObjectType then
                self.dataFrame = frame
            else
                self.RefreshButton:StopRefresh()
                self.dataFrame = nil
                local message = L["ErrorNotAUiObject"]:format(self.FrameInput:GetText())
                DEFAULT_CHAT_FRAME:AddMessage(message, 0.8, 0.1, 0,1)
            end
        end
    end
    if self.dataFrame then
        self.DisplayBase.Frames[self.DisplayBase.SelectedTab]:LoadFrame(self.dataFrame, forceRefresh)
    else
        self.RefreshButton:StopRefresh()
        self:ClearValue()
    end
end


function Public.ClearValue(self)
    for _,tab in pairs(self.DisplayBase.Frames) do
        tab:ClearValue()
    end
end


function Scripts.OnDragStart(self, mouseButton)
    self:StartMoving()
end


function Scripts.OnDragStop(self)
    self:StopMovingOrSizing()
    FrameDisplay.Manager:Save(self)
end


function Scripts.OnHide(self)
    FrameDisplay.Manager:Save(self)
end


function Scripts.OnShow(self)
    FrameDisplay.Manager:Save(self)
end


function ScriptsFrameInput.OnEnterPressed(self)
    local frame = self:GetText()
    FrameDisplay.Manager:Save(self:GetParent())
    self:GetParent().dataFrame = nil
    self:GetParent():RefreshData(true)
end


function ScriptsDelete.OnClick(self, mouseButton, isDown)
    local display = self:GetParent()
    display.FrameInput:SetText("")
    display.dataFrame = nil
    display:Hide()
end


function ScriptsDisplayTabs.OnShow(self)
    local displayFrame = self:GetParent():GetParent()
    -- once one tab has had data displayed, all tabs will auto
    -- display data when selected
    displayFrame:RefreshData(displayFrame.dataFrame ~= nil)
end


function Private.Initialize()
    DataDumpData = DataDumpData or {}
    FrameDisplay.Manager = DisplayManager.Init(DataDumpData, "FrameDisplay", Private.Create)
end


LazyLib.MakeLazy(FrameDisplay, Private.Initialize)
