-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local getmetatable = getmetatable
local ipairs = ipairs
local pairs = pairs
local tostring = tostring
local type = type
-- blizzard api
local CreateFrame = CreateFrame
-- libs
-- add on api
local L = DataDump.Localization()
local FrameDisplay = DataDump.NameSpace("FrameDisplay", "Display")
local PointEditor = DataDump.NameSpace("FrameDisplay", "Editor", "Point")
local Simple = DataDump.NameSpace("FrameDisplay", "Editor", "Simple")
local VarData = DataDump.NameSpace("VarDisplay", "Data")
local VarDisplay = DataDump.NameSpace("VarDisplay", "Display")
-- local name space
local RegionTab = DataDump.NameSpace("FrameDisplay", "Region")
local Public = {}
local Rect = {}
local ScriptPoints = {}
local ScriptGetChildren = {}
local ScriptGetRegions = {}
local ScriptGetFunctions = {}
local ScriptGetParent = {}
local ScriptsResizable = {}
local ScriptClamped = {}
local Private = {}
-- locals


function RegionTab.Create(parent)
    local name = parent:GetName().."RegionTab"
    local displayTab = CreateFrame("Frame", name, parent, "DataDumpRegionTabTemplate")
    displayTab:SetAllPoints()
    displayTab:SetFrameLevel(parent:GetFrameLevel()+1)

    displayTab.CoordinateType:SetScale(0.75)
    displayTab.CoordinateType.MenuInfo = Private.CreateCoordinateMenu(displayTab)
    displayTab.CoordinateType:SelectValue("Coordinates")
    displayTab.CoordinateType:SetPoint("BottomRight", displayTab.Positions, "TopRight")

    -- points
    for _,line in ipairs(displayTab.Points.Point) do
        PointEditor.InitializePointLine(line, displayTab.PointEditor)
        line:Hide()
    end
    displayTab.Points:SetScript("OnMouseWheel", ScriptPoints.OnMouseWheel)
    displayTab.Points.Scroll:SetCallback(Private.ScrollOnValueChanged, displayTab.Points)
    displayTab.Points.Scroll.__HideOnDisable = true

    PointEditor.InitializePointEditor(displayTab.PointEditor)
    displayTab.PointEditor.displayTab = displayTab

    displayTab.GetChildren:SetText(L["Child Frames"])
    for k,v in pairs(ScriptGetChildren) do
        displayTab.GetChildren:SetScript(k, v)
    end

    displayTab.GetRegions:SetText(L["Child Regions"])
    for k,v in pairs(ScriptGetRegions) do
        displayTab.GetRegions:SetScript(k, v)
    end

    displayTab.GetFunctions:SetText(L["GetFunctions"])
    for k,v in pairs(ScriptGetFunctions) do
        displayTab.GetFunctions:SetScript(k, v)
    end

    displayTab.GetParentFrame:SetText(L["GetParent"])
    for k,v in pairs(ScriptGetParent) do
        displayTab.GetParentFrame:SetScript(k, v)
    end

    -- settings
    Rect.Init(displayTab.Positions)
    Simple.InitializeNodes(displayTab.Settings)
    displayTab.refreshList = {}
    displayTab.Settings.IsResizable:SetScript("OnClick", ScriptsResizable.OnClick)
    displayTab.Settings.IsClampedToScreen:SetScript("OnClick", ScriptClamped.OnClick)

    for k,v in pairs(Public) do
        displayTab[k] = v
    end

    return displayTab
end


function Public.LoadFrame(self, dataFrame, forceRefresh)
    self.dataFrame = dataFrame or self.dataFrame

    if self.dataFrame then
        if self.dataFrame ~= self.enabledDataFrame or forceRefresh then
            self.refreshList.Rect = self.Positions:SetEnable(self.dataFrame)
            Simple.GetRefreshList(self.Settings, self.dataFrame, self.refreshList)
            self.GetChildren:SetEnabled(self.dataFrame.GetChildren ~= nil)
            self.GetRegions:SetEnabled(self.dataFrame.GetRegions ~= nil)
            self.GetFunctions:Enable()
            self.GetParentFrame:Enable()
            self.Points.__enable = true
            if self.dataFrame == UIParent then
                self.GetParentFrame:Disable()
                self.Points.__enable = false
                self.Positions:Disable()
                Simple.DisableAll(self.Settings)
            end
            self.enabledDataFrame = self.dataFrame
        end
        Simple.Refresh(self.refreshList)

        Private.UpdatePoints(self.Points, self.dataFrame, 1)

    end
end


function Public.ClearValue(self)
    Simple.ClearAllValues(self.Settings)
    self.Positions:ClearValue()
    self.GetChildren:Disable()
    self.GetRegions:Disable()
    self.GetFunctions:Disable()
    self.GetParentFrame:Disable()
    self.enabledDataFrame = nil
end


do -- Rect node
    local function RefreshRect(node)
        local left, bottom, width, height = node.dataFrame:GetRect()
        left = left or 0
        bottom = bottom or 0
        width = width or 0
        height = height or 0
        node.Top:SetText(Simple.ReduceNumber2(bottom + height))
        node.Bottom:SetText(Simple.ReduceNumber2(bottom))
        node.Left:SetText(Simple.ReduceNumber2(left))
        node.Right:SetText(Simple.ReduceNumber2(left + width))
        node.Width.Input:SetText(Simple.ReduceNumber2(width))
        node.Height.Input:SetText(Simple.ReduceNumber2(height))

        local centerX, centerY = node.dataFrame:GetCenter()
        node.CenterX.Value:SetText(Simple.ReduceNumber2(centerX))
        node.CenterY.Value:SetText(Simple.ReduceNumber2(centerY))
    end


    local function RefreshBounds(node)
        local left, bottom, width, height = node.dataFrame:GetBoundsRect()
        node.Top:SetText(Simple.ReduceNumber2(bottom + height))
        node.Bottom:SetText(Simple.ReduceNumber2(bottom))
        node.Left:SetText(Simple.ReduceNumber2(left))
        node.Right:SetText(Simple.ReduceNumber2(left + width))
        node.Width.Input:SetText(Simple.ReduceNumber2(width))
        node.Height.Input:SetText(Simple.ReduceNumber2(height))

        node.CenterX.Value:SetText(Simple.ReduceNumber2(bottom + (height / 2)))
        node.CenterY.Value:SetText(Simple.ReduceNumber2(left + (width / 2)))
    end


    local function RectClearValue(node)
        node.Top:SetText("")
        node.Bottom:SetText("")
        node.Left:SetText("")
        node.Right:SetText("")
        node.Width.Input:SetText("")
        node.Height.Input:SetText("")
        node.CenterX.Value:SetText("")
        node.CenterY.Value:SetText("")
        node.dataFrame = nil
    end


    local function RectDisable(node)
        node.Width.Input:Disable()
        node.Height.Input:Disable()
    end


    local function RectSetEnable(node, dataFrame)
        if node.getBounds and dataFrame.GetBoundsRect then
            node.Width.Input:Disable()
            node.Height.Input:Disable()
            node.Refresh = RefreshBounds
            node.dataFrame = dataFrame
            return node
        elseif not node.getBounds and dataFrame.GetRect then
            node.Width.Input:Enable()
            node.Width.Input.dataFrame = dataFrame
            node.Height.Input:Enable()
            node.Height.Input.dataFrame = dataFrame
            node.Refresh = RefreshRect
            node.dataFrame = dataFrame
            return node
        else
            RectClearValue(node)
        end
    end


    function Rect.Init(node)
        node.SetEnable = RectSetEnable
        node.ClearValue = RectClearValue
        node.Disable = RectDisable
        node.Width.Title:SetText(L["Simple"]["GetWidth"])
        node.Width.Input.isNumber = true
        node.Width.Input.Callback = "SetWidth"
        node.Height.Title:SetText(L["Simple"]["GetHeight"])
        node.Height.Input.isNumber = true
        node.Height.Input.Callback = "SetHeight"
        node.CenterX.Title:SetText(L["Center X"])
        node.CenterY.Title:SetText(L["Center Y"])
        node.Refresh = RefreshRect
    end
end -- Rect node


function Private.CreateCoordinateMenu(displayTab)
    return
    {
        { Text=L["Coordinates"], Value="Coordinates", Func=Private.MenuOnSelect, Tab=displayTab },
        { Text=L["Bounds"],      Value="Bounds",      Func=Private.MenuOnSelect, Tab=displayTab },
    }
end


function Private.MenuOnSelect(info, mouseButton, isDown, oldInfo)
    if info.Value == "Coordinates" then
        info.Tab.Positions.getBounds = nil
    else
        info.Tab.Positions.getBounds = true
    end
    info.Tab.enabledDataFrame = nil -- force full refresh
    info.Tab:LoadFrame(nil, true)
end

-- todo: move all point frame code over to Point file
-- todo: create SetEnable function
function Private.UpdatePoints(pointsFrame, dataFrame, offset)
    local lineIndex = 1
    local count = 0
    if pointsFrame.__enable then
        dataFrame = dataFrame or pointsFrame.dataFrame
        pointsFrame.dataFrame = dataFrame
        offset = offset or pointsFrame.Scroll:GetScrollValue()
        count = dataFrame and dataFrame:GetNumPoints() or 0
        local dataIndex = offset
        while lineIndex <= 4 and dataIndex <= count do
            pointsFrame.Point[lineIndex]:SetValue(dataFrame, dataIndex):Show()
            lineIndex = lineIndex + 1
            dataIndex = dataIndex + 1
        end
        if lineIndex <= 4 then
            pointsFrame.Point[lineIndex]:NewLine(dataFrame, lineIndex):Show()
            lineIndex = lineIndex + 1
        end
    end
    while lineIndex <= 4 do
        pointsFrame.Point[lineIndex]:Hide()
        lineIndex = lineIndex + 1
    end
    pointsFrame.Scroll:UpdateScrollRange(4, count, offset)
end


function ScriptPoints.OnMouseWheel(self, delta)
    self.Scroll:Scroll(delta)
end


function Private.ScrollOnValueChanged(scrollPosition, pointsFrame)
    Private.UpdatePoints(pointsFrame, nil, Floor(scrollPosition))
end


function ScriptGetChildren.OnClick(self, mouseButton, isDown)
    local dataFrame = self:GetParent().dataFrame
    if dataFrame and dataFrame.GetChildren then
        local varDisplay = VarDisplay.Manager:GetDisplay()
        local name = (dataFrame:GetName() or tostring(dataFrame))..":GetChildren()"
        varDisplay.VarInput:SetText(name)
        varDisplay.VarInput:Disable()
        varDisplay.noSave = true
        local f = function()
            return { dataFrame:GetChildren() }
        end
        local varData = VarData.CreateWithFunction(f, name)
        varDisplay:SetCrumb(varData, 1)
        varDisplay:Show()
    end
end


function ScriptGetRegions.OnClick(self, mouseButton, isDown)
    local dataFrame = self:GetParent().dataFrame
    if dataFrame and dataFrame.GetChildren then
        local varDisplay = VarDisplay.Manager:GetDisplay()
        local name = (dataFrame:GetName() or tostring(dataFrame))..":GetRegions()"
        varDisplay.VarInput:SetText(name)
        varDisplay.VarInput:Disable()
        varDisplay.noSave = true
        local f = function()
            return { dataFrame:GetRegions() }
        end
        local varData = VarData.CreateWithFunction(f, name)
        varDisplay:SetCrumb(varData, 1)
        varDisplay:Show()
    end
end


function ScriptGetFunctions.OnClick(self, mouseButton, isDown)
    local dataFrame = self:GetParent().dataFrame
    if dataFrame then
        local functionList = {}
        local meta = getmetatable(dataFrame)
        for k,v in pairs(meta.__index) do
            functionList["•"..k] = L["FunctionMeta"]
        end
        for k,v in pairs(dataFrame) do
            if type(v) == "function" then
                if meta.__index[k] then
                    functionList["•"..k] = L["FunctionOverride"]
                else
                    functionList["•"..k] = L["FunctionAttached"]
                end
            end
        end

        local name = (dataFrame:GetName() or tostring(dataFrame)).." Function List"

        local f = function()
            return functionList
        end
        local varData = VarData.CreateWithFunction(f, name)

        local varDisplay = VarDisplay.Manager:GetDisplay()
        varDisplay.VarInput:SetText(name)
        varDisplay.VarInput:Disable()
        varDisplay.noSave = true
        varDisplay:SetCrumb(varData, 1)
        varDisplay:Show()
    end
end


function ScriptGetParent.OnClick(self, mouseButton, isDown)
    local dataFrame = self:GetParent().dataFrame
    if dataFrame then
        local parent = dataFrame:GetParent()
        local name = parent and parent:GetName() or L["•unnamed"]
        local frameDisplay = FrameDisplay.Manager:GetDisplay()
        frameDisplay:SetFrameInput(name, parent)
        frameDisplay:RefreshData(true)
        frameDisplay:Show()
    end
end


function ScriptsResizable.OnClick(self, mouseButton, isDown)
    if not self.Editor then
        self.Editor = CreateFrame("Frame", self:GetName().."Editor", self, "DataDumpResizableEditorTemplate")
        self.Editor:SetPoint("TopLeft", self, "BottomLeft", 0, -20)
        self.Editor.OnOff.Title:SetText(L["Simple"]["IsResizable"])
        self.Editor.MinX.Title:SetText(L["MinX"])
        self.Editor.MinY.Title:SetText(L["MinY"])
        self.Editor.MaxX.Title:SetText(L["MaxX"])
        self.Editor.MaxY.Title:SetText(L["MaxY"])
        self.Editor.Save:SetText(L["Set"])
        self.Editor.Cancel:SetText(L["Cancel"])
        self.Editor.Refresh = Private.RefreshResizableEditor
        self.Editor:Hide()
    end
    if self.Editor:IsShown() then
        self.Editor.saveResults = nil
        self.Editor:Hide()
    else
        self.Editor.dataFrame = self.dataFrame
        self.Editor:Refresh()
        self.Editor:Show()
    end
end


function Private.RefreshResizableEditor(editor)
    if editor.dataFrame then
        editor.OnOff.Check:SetChecked(editor.dataFrame:IsResizable())
        local minX, minY = editor.dataFrame:GetMinResize()
        local maxX, maxY = editor.dataFrame:GetMaxResize()
        editor.MinX.Input:SetText(Simple.ReduceNumber2(minX or 0))
        editor.MinY.Input:SetText(Simple.ReduceNumber2(minY or 0))
        editor.MaxX.Input:SetText(Simple.ReduceNumber2(maxX or 0))
        editor.MaxY.Input:SetText(Simple.ReduceNumber2(maxY or 0))
    end
end


function ScriptClamped.OnClick(self, mouseButton, isDown)
    if not self.Editor then
        self.Editor = CreateFrame("Frame", self:GetName().."Editor", self, "DataDumpClampEditorTemplate")
        self.Editor:SetPoint("TopLeft", self, "BottomLeft", 0, 0)
        self.Editor.OnOff.Title:SetText(L["Clamped"])
        self.Editor.Top.Title:SetText(L["top"])
        self.Editor.Left.Title:SetText(L["left"])
        self.Editor.Right.Title:SetText(L["right"])
        self.Editor.Bottom.Title:SetText(L["bottom"])
        self.Editor.Save:SetText(L["Set"])
        self.Editor.Cancel:SetText(L["Cancel"])
        self.Editor.Refresh = Private.RefreshClampedEditor
        self.Editor:Hide()
    end
    if self.Editor:IsShown() then
        self.Editor.saveResults = nil
        self.Editor:Hide()
    else
        self.Editor.dataFrame = self.dataFrame
        self.Editor:Refresh()
        self.Editor:Show()
    end
end


function Private.RefreshClampedEditor(editor)
    if editor.dataFrame then
        local isClamped = editor.dataFrame:IsClampedToScreen()
        editor.OnOff.Check:SetChecked(isClamped)
        local left, right, top, bottom = editor.dataFrame:GetClampRectInsets()
        editor.Left.Input:SetText(Simple.ReduceNumber2(left or 0))
        editor.Right.Input:SetText(Simple.ReduceNumber2(right or 0))
        editor.Top.Input:SetText(Simple.ReduceNumber2(top or 0))
        editor.Bottom.Input:SetText(Simple.ReduceNumber2(bottom or 0))
    end
end
