-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
-- this convoluted structure is used to track the values and subvalues of a tables
-- the purpose is to retain the structure of the tracked data, without actually modifying
-- the data.  Furthermore, it is designed to not constantly reallocate tables so that
-- it uses a constant amount of memory.  This is really important if data is being
-- auto-refreshed during combat, since the client turns garbage collection off.
local _, DataDump = ...
-- lua api
local TableSort = table.sort
local ipairs = ipairs
local loadstring = loadstring
local pcall = pcall
local setmetatable = setmetatable
local type = type
-- blizzard api
local Wipe = wipe
-- libs
-- add on api
-- local name space
local VarData = DataDump.NameSpace("VarDisplay", "Data")
local VarDataMeta = {}; VarDataMeta.__index = VarDataMeta
local Private = {}
-- locals
local _kIndent = 15


function VarData.Create(key)
    local executeValue = "return "..key
    local func, err = loadstring(executeValue)
    if func then
        local data =
        {
            key = key,
            func = func,
        }
        return setmetatable(data, VarDataMeta)
    end
end


-- key is optional
function VarData.CreateWithFunction(func, key)
    return setmetatable({ func = func, key = key }, VarDataMeta)
end


-- key is optional
function VarData.CreateWithValue(value, key)
    return setmetatable({ value = value, key = key }, VarDataMeta)
end


function VarDataMeta.Copy(self)
    local newData = {}
    for k,v in pairs(self) do
        newData[k] = v
    end
    newData.subValues = nil
    newData.subValuesSorted = nil
    newData.shouldExpand = nil
    return setmetatable(newData, VarDataMeta)
end


function VarDataMeta.Eval(self, forceRefresh)
    if forceRefresh or self.value == nil then
        self.parseError = nil
        if self.func then
            local success, value = pcall(self.func)
            if success then
                self.value = self.func()
            else
                DEFAULT_CHAT_FRAME:AddMessage("Failed to parse:"..tostring(self.key))
                DEFAULT_CHAT_FRAME:AddMessage(value)
                self.parseError = true
            end
        elseif self.parentData then
            self.value = self.parentData[self.key]
        end
        if self.shouldExpand then
            Private.Expand(self)
        end
    end
    return self
end


function VarDataMeta.Expand(self, forceRefresh)
    self.shouldExpand = nil
    self:Eval(forceRefresh)
    self.shouldExpand = Private.Expand(self)
    return self
end


function VarDataMeta.Collapse(self, forceRefresh)
    self.shouldExpand = nil
    self:Eval(forceRefresh)
    return self
end


function VarDataMeta.FlattenOutput(self, forceRefresh)
    self:Eval(forceRefresh)
    self.output = Wipe(self.output or {})
    Private.AppendDisplayData(self.output, self, 0)
    return self.output
end


function VarDataMeta.GetFullName(self, forceRefresh)
    self:Eval(forceRefresh)
    if self.parseError then
        return nil
    end
    if self.parent then
        local keyString = Private.GetKeyString(self)
        if keyString then
            local parentName = VarDataMeta.GetFullName(self.parent, forceRefresh)
            if parentName then
                return parentName..keyString
            end
        end
    else
        return Private.CheckKeySafety(self.key) and self.key or nil
    end
    return nil
end


function VarDataMeta.GetRegionName(self, forceRefresh)
    self:Eval(forceRefresh)
    if self.parseError then
        return nil
    end
    if self.value.IsObjectType and self.value:IsObjectType("Region") then
        local name = self.value:GetName()
        if not name and self.parent then
            local keyString = Private.GetKeyString(self)
            if keyString then
                local parentName = VarDataMeta.GetRegionName(self.parent, forceRefresh)
                if parentName then
                    return parentName..keyString
                end
            end
        end
        return name
    end
    return nil
end


function Private.Expand(self)
    if type(self.value) == "table" then
        self.subValues = self.subValues or {}
        self.subValuesSorted = Wipe(self.subValuesSorted or {})
        local index = 1
        for k,v in pairs(self.value) do
            local sub = self.subValues[k] or setmetatable({ parent = self, key = k }, VarDataMeta)
            sub.value = v
            sub.parentData = self.value

            if sub.shouldExpand then
                Private.Expand(sub)
            end

            self.subValues[k] = sub
            self.subValuesSorted[index] = sub
            index = index + 1
        end
        TableSort(self.subValuesSorted, Private.SortByKey)
        return true
    end
    return nil
end


function Private.SortByKey(lt, rt)
    if type(lt.key) == "number" then
        lt = lt.key
    else
        lt = tostring(lt.key)
    end
    if type(rt.key) == "number" then
        rt = rt.key
    else
        rt = tostring(rt.key)
    end
    if type(lt) == "string" then
        if type(rt) == "number" then
            return true
        end
    -- lt is number
    elseif type(rt) == "string" then
        return false
    end
    return lt < rt
end


function Private.AppendDisplayData(output, varData, indent)
    varData.indent = indent
    output[#output+1] = varData
    if type(varData.value) == "table" and varData.shouldExpand then
        for _,subValue in ipairs(varData.subValuesSorted) do
            Private.AppendDisplayData(output, subValue, indent + _kIndent)
        end
    end
end


function Private.GetKeyString(self)
    if type(self.key) == "boolean"
    or type(self.key) == "number" then
        return "["..tostring(self.key).."]"
    end
    if type(self.key) == "string"
    and Private.CheckKeySafety(self.key) then
        return "."..self.key
    end
    return nil
end


function Private.CheckKeySafety(key)
    local from,to = key:find("[_a-zA-Z][_a-zA-Z0-9]*")
    return from == 1 and to == key:len()
end
